﻿#define TRACE



using AIOUSBNet;  // namespace exposes AIOUSB Class interface
using Sharp7;
using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
namespace Sample1
{
    public partial class ConsoleTestForm3 : Form
    {

        public TConsoleResultData ManifestData = new TConsoleResultData();
        public PLC PLCInterface = new PLC();

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool ShowWindowAsync(IntPtr hWnd, int nCmdShow);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool ShowWindow(IntPtr hWnd, int nCmdShow);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool SetWindowLongPtrA(IntPtr hWnd, int nIndex, IntPtr dwNewLong);

        [DllImport("user32.dll")]
        private static extern IntPtr SetActiveWindow(IntPtr hWnd);

        [DllImport("user32.dll", EntryPoint = "SetWindowPos")]
        public static extern IntPtr SetWindowPos(IntPtr hWnd, int hWndInsertAfter, int x, int Y, int cx, int cy, int wFlags);

        [DllImport("user32.dll", EntryPoint = "SetForegroundWindow")]
        public static extern bool SetForegroundWindow(IntPtr hWnd);
        private const int SWP_NOSIZE = 0x0001;
        private const int SWP_NOMOVE = 0x0002;
        private const int SWP_NOZORDER = 0x0004;
        private const int SWP_NOREDRAW = 0x0008;
        private const int SWP_NOACTIVATE = 0x0010;
        private const int SWP_FRAMECHANGED = 0x0020;
        private const int SWP_SHOWWINDOW = 0x0040;
        private const int SWP_HIDEWINDOW = 0x0080;
        private const int SWP_NOCOPYBITS = 0x0100;
        private const int SWP_NOOWNERZORDER = 0x0200;
        private const int SWP_NOSENDCHANGING = 0x0400;
        private const int SWP_DEFERERASE = 0x2000;
        private const int SWP_ASYNCWINDOWPOS = 0x4000;
        private const int SW_SHOWMINNOACTIVE = 7;

        private Process ProcessProperties = new Process();

        public delegate void frm3Focus();
        public frm3Focus myDelegate;

        private S7Client Client = new S7Client();
        private Socket sock;

        public int USBTest { get; private set; }

        //Declare and Initialize the IP Adress
        private readonly IPAddress ipAd = IPAddress.Parse("10.11.51.222");

        private bool bVSpyDone;
        private bool b5vUSBwasDisconnected;

        public struct ComponentResult
        {
            public string SerialNumber; // Component Serial Number
            public int TestResult; // Result code 0:Unknown, 1:Good, 2:Scrap
            public double LeakDetected; // Leak value [cc/min]
            public DateTime TestDateTime; // Test Timestamp
        }

        public ConsoleTestForm3()
        {
            InitializeComponent();
        }

        //**************************************************************
        //**********  Form 3 Load **************************************
        //**************************************************************
        private void ConsoleTestForm_Load(object sender, EventArgs e)
        {
            Trace.WriteLine("BEGIN ConsoleTestForm_Load()");
            Trace.Indent();

            const string errPLC = "PLC Not Ready";

            if (!PLC.bPLCReady() && OperatorInstructions.Text != errPLC)
            {
                OperatorInstructions.Text = errPLC;
                Trace.TraceError(errPLC);
            }

            while (!(PLC.bGetStart() || cbFakePLCStart.Checked)) Application.DoEvents();

            myDelegate = new ConsoleTestForm3.frm3Focus(EnsureTopmostFocused);

            if (gv.ConsoleTestResultForm != null)
                gv.ConsoleTestResultForm.Manifest_Options.Text = RetrieveManifestFromPLC();
            if (gv.HeadlinerTestResultsOptionsForm != null)
                gv.HeadlinerTestResultsOptionsForm.Manifest_Options.Text = RetrieveManifestFromPLC();

            InitializePartNumberTextBoxes();

            /* if manifest has not arrived then show that on the form
             *
             *
             * */
            gv.CUPHOLDER_USB_SCAN = gv.CUPHOLDER_USB_TEST = (ManifestData.HeadModcode[4301] != "") ? "YELO" : "GRAY";// Search Manifest for Modcode (Cupholder USB)
            gv.BODY_USB_SCAN = gv.BODY_USB_TEST = (ManifestData.HeadModcode[3002] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (Body USB)
            gv.ENDCAP_USB_SCAN = gv.ENDCAP_USB_TEST = (ManifestData.HeadModcode[1502] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (Endcap USB)

            InitDisplayBasedOnManifest();

            if (gv.diDIO == AIOUSB.diNone)
            {
                // Handle no IIRO / IDIO device found
            }
            else if (gv.RETEST != "true")
            {
                // Initialize all Outputs to the OFF State
                _ = AIOUSB.DIO_Write1(gv.diDIO, 6, 1);  // Deactivate TEST FAIL (Interlock)
                _ = AIOUSB.DIO_Write1(gv.diDIO, 7, 1);  // Deactivate TEST PASS (Interlock)
                _ = AIOUSB.DIO_Write1(gv.diDIO, 8, 1);  // Deactivate END OF CYCLE (Interlock)
                _ = AIOUSB.DIO_Write1(gv.diDIO, 9, 1);  // Turn OFF SOL #1   CONSOLE 55-way LATCH / HEADLINER LATCH BH#1
                _ = AIOUSB.DIO_Write1(gv.diDIO, 10, 1); // Turn OFF SOL #2   CONSOLE 55-way DRAW DOWN / HEADLINER LATCH BH#2
                _ = AIOUSB.DIO_Write1(gv.diDIO, 11, 1); // Turn OFF SOL #3   HEADLINER DRAW DOWN BH #1
                _ = AIOUSB.DIO_Write1(gv.diDIO, 12, 1); // Turn OFF SOL #4   HEADLINER DRAW DOWN BH #2
                _ = AIOUSB.DIO_Write1(gv.diDIO, 13, 1); // TURN OFF SOL #5   HEADLINER AIR EJECT BH #3
                _ = AIOUSB.DIO_Write1(gv.diDIO, 14, 1); // CONSOLE (LED) RFID TEST BLOCK  5VdcUSB & 12Vdc Power Outlet

            }
            Timer1.Start();

            Trace.WriteLine("ConsoleConnectTestForm3 END");
        }

        private void buttonWriteDefaultManifestToPLC_Click(object sender, EventArgs e)
        {
            //*************************************************************
            // ***** PREMIUM MANIFEST FOR RED RABBIT TEST NOT FROM THE PLC
            //****** MUST REMOVE REMARKS FOR PRODUCTION
            //*************************************************************

            // CONSOLE MANIFEST - Highline (default)                                                          
            // remove this comment to update PLC with manual manifest
            PLCInterface.WriteManifest(Encoding.ASCII.GetBytes("0000000355149,103xxxxxxxxxx,0000004364960,13525890FT12,1502,13529874FW86,3002,13529875FW85,4301,135619101FT06,402,13592451FT45,1302,85117464FT09,1101,84638621GO2A,2701,13592452FW74,3501,84952174FW83,4201,84786104FW68,3101,84960563M33V,3301,13537143M08L,2901,"));
        }

        public string RetrieveManifestFromPLC()
        {
            Trace.WriteLine("BEGIN RetrieveManifestFromPLC()");
            Trace.Indent();
            string manifest = PLCInterface.GetManifest();
            string[] manifestTokens = manifest.Split(',');

            ManifestData.Clear();
            ManifestData.BuildID = manifestTokens[0];
            ManifestData.CartNumber = manifestTokens[1];
            ManifestData.CSN = manifestTokens[2];

            for (int token = 3; token < manifestTokens.Length - 1; token += 2)
            {
                int modcode = int.Parse(manifestTokens[token + 1]);
                if (modcode < 9999)
                    ManifestData.HeadModcode[modcode] = manifestTokens[token];
                else
                {
                    Trace.TraceError("Error parsing manifest string from PLC: modcode was " + modcode.ToString() + " and PN was " + manifestTokens[token]);
                }
            }
            Trace.Unindent();
            Trace.WriteLine("END RetrieveManifestFromPLC()");
            return manifest;
        }

        public void InitDisplayBasedOnManifest()
        {
            Trace.WriteLine("InitDisplayBasedOnManifest()");
            // *******************************************************
            // ********* CONSOLE TEST MOD codes **********************
            // *******************************************************
            gv.EOCM = (ManifestData.HeadModcode[3301] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (EOCM)
            gv.WCM = (ManifestData.HeadModcode[2901] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (WCM - WIRELESS CHARGE MODULE)
            gv.SEAT = (ManifestData.HeadModcode[1101] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (HEATED SEAT SW.)
            gv.PEPS = (ManifestData.HeadModcode[1302] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (PEPS ANTENNA)
            gv.SIB = (ManifestData.HeadModcode[2701] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (SIB - SHIFT INTERFACE BOARD)
            gv.TBS = (ManifestData.HeadModcode[4201] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (TRAILER BRAKE SWITCH)
            gv.LAMP = (ManifestData.HeadModcode[402] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (MAIN CONSOLE LAMP)
            gv.IMOB = (ManifestData.HeadModcode[3501] != "") ? "YELO" : "GRAY"; // Search Manifest for Modcode (IMMOBILIZER ANTENNA)
            gv.HVPO = ((ManifestData.HeadModcode[3101] != "")
                    || (ManifestData.HeadModcode[3102] != "")) ? "YELO" : "GRAY"; // Search Manifest for Modcode (HIGH VOLTAGE PO 3101)

            OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
            OperatorInstructions.Text += "FIRST CONNECT USB\n\r\n\r\n\r";
            OperatorInstructions.Text += "SECOND CONNECT 55-WAY\n\r\n\r";

            //***********************************************************************************************************
            //************* iNITIALIZE WHICH TEST TO RUN BASED ON mod CODE OR MANUAL SELECTIONS GLOBAL VARIABLES = "YELO"
            //************************************************************************************************************
            button_LIN_Heated_Seat_Switch.BackColor = gv.SEAT.Contains("YELO") ? Color.Yellow : Color.LightGray;
            button_LIN_TBS.BackColor = gv.TBS.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonWCM.BackColor = gv.WCM.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonSIB.BackColor = gv.SIB.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonEOCM.BackColor = gv.EOCM.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonRFIDendcapUSB.BackColor = gv.ENDCAP_USB_SCAN.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonTESTendcapUSB.BackColor = gv.ENDCAP_USB_TEST.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonRFIDbodyUSB.BackColor = gv.BODY_USB_SCAN.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonTESTbodyUSB.BackColor = gv.BODY_USB_TEST.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonRFIDcupHolderUSB.BackColor = gv.CUPHOLDER_USB_SCAN.Contains("YELO") ? Color.Yellow : Color.LightGray;
            buttonTESTcupHolderUSB.BackColor = gv.CUPHOLDER_USB_TEST.Contains("YELO") ? Color.Yellow : Color.LightGray;
        }

        private void InitializePartNumberTextBoxes()
        {
            Trace.WriteLine("InitializePartNumberTextBoxes()");
            textBoxfrm3VIN.Text = gv.VIN;

            // ResultData.bInitalizePartNumberTextBoxes = true;

            textBoxEOCM_BaseModelPartNumber8.Text = "";
            textBoxEOCM_EndModelPartNumber8.Text = "";
            textBoxEOCM_BootSWID8.Text = "";
            textBoxEOCM_AppSWID10.Text = "";
            textBoxEOCM_AppDataID10.Text = "";

            textBoxSIB_BaseModelPartNumber8.Text = "";
            textBoxSIB_EndModelPartNumber8.Text = "";
            textBoxSIB_BootSWID8.Text = "";
            textBoxSIB_AppSWID10.Text = "";
            textBoxSIB_AppDataID10.Text = "";

            textBoxWCM_BaseModelPartNumber8.Text = "";
            textBoxWCM_EndModelPartNumber8.Text = "";
            textBoxWCM_BootSWID8.Text = "";
            textBoxWCM_AppSWID10.Text = "";
            textBoxWCM_AppDataID10.Text = "";

            TextBox_Time_of_PN_Read.Text = "";
            TextBox_Date_of_PN_Read.Text = "";
        }
        /**********************************************************************************************************/
        private void timerTest_Tick(object sender, EventArgs e)
        {
            int USBTestFailCount;
            textBoxfrm3VIN.Text = gv.VIN;

            PLC.Heartbeat();

            PollInputs(out bool bConsole_PIP_55Way_Detected, out bool b5VUSBDetected);

            if (bConsole_PIP_55Way_Detected)
            {
                // ONE SHOT
                if (!ManifestData.b55_WayConnected)
                    StartTest();

                if (true)
                {
                    bVSpyDone = vSpyGatherPartNumbers_StateMachine(); // state machine, runs every tick to perform next step in process
                }

                HandleCardScan();

                switch (USBTest)
                {
                    case 0:
                        break;
                    case 2:
                        TestUSB_Cupholder(b5VUSBDetected);
                        break;
                    case 4:
                        TestUSB_Body(b5VUSBDetected);
                        break;
                    case 7:
                        TestUSB_Endcap(b5VUSBDetected);
                        break;
                }

                USBTestFailCount = gv.CUPHOLDER_USB_TEST.Contains("YELO") ? 1 : 0;
                USBTestFailCount += gv.BODY_USB_TEST.Contains("YELO") ? 1 : 0;
                USBTestFailCount += gv.ENDCAP_USB_TEST.Contains("YELO") ? 1 : 0;
                ButtonAbortTest.Visible = bVSpyDone && (USBTestFailCount > 0);

                //if (global_variables.WCM.Contains("PASS"))
                //if ( (ResultData.HeadModcode[3101] != "") || (ResultData.HeadModcode[3102] != ""))
                if (USBTestFailCount == 0)
                {
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text += "<<< TESTER >>>\n\r\n\r\n\r";
                    OperatorInstructions.Text += "READING MODULE\n\r\n\r\n\r";
                    OperatorInstructions.Text += "PART NUMBERS\n\r\n\r\n\r";

                    ManifestData.bAllRequiredUSBsWereTested = true;
                    if (ManifestData.bParsedMultipleTXTFiles)
                    {
                        Timer1.Stop();
                        gv.DiagnosticForm.StartADCReadings();
                        gv.DiagnosticForm.Show();
                    }
                }

                if (bVSpyDone)
                {
                    OperatorInstructions.Text += "Generating Results\r\n\r\n";

                    if (!ManifestData.ParseComplete) ParseVSpyResultsFiles();

                }

                if ((USBTestFailCount == 0) && (bVSpyDone))
                {
                    Trace.WriteLine("CycleComplete in Timer Tick");
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text += "<<< TESTER >>>\n\r\n\r\n\r";
                    OperatorInstructions.Text += "Test Complete\n\r\n\r\n\r";
                }
            }
        }

        private void HandleCardScan()
        {
            if (OperatorInstructions.Text.Contains("7"))
            {

                if ((USBTest == 4) && (gv.BODY_USB_TEST != "PASS"))
                {
                    gv.BODY_USB_SCAN = "YELO";
                    buttonRFIDbodyUSB.BackColor = Color.Yellow;
                }
                else if ((USBTest == 2) && (gv.CUPHOLDER_USB_TEST != "PASS"))
                {
                    gv.CUPHOLDER_USB_SCAN = "YELO";
                    buttonRFIDcupHolderUSB.BackColor = Color.Yellow;
                }

                if (!gv.ENDCAP_USB_SCAN.Contains("GRAY")) USBTest = 7;
            }
            if (OperatorInstructions.Text.Contains("4"))
            {
                if ((USBTest == 7) && (gv.ENDCAP_USB_TEST != "PASS"))
                {
                    gv.ENDCAP_USB_SCAN = "YELO";
                    buttonRFIDendcapUSB.BackColor = Color.Yellow;
                }
                else if ((USBTest == 2) && (gv.CUPHOLDER_USB_TEST != "PASS"))
                {
                    gv.CUPHOLDER_USB_SCAN = "YELO";
                    buttonRFIDcupHolderUSB.BackColor = Color.Yellow;
                }
                if (!gv.BODY_USB_SCAN.Contains("GRAY")) USBTest = 4;
            }
            if (OperatorInstructions.Text.Contains("2"))
            {
                if ((USBTest == 4) && (gv.BODY_USB_TEST != "PASS"))
                {
                    gv.BODY_USB_SCAN = "YELO";
                    buttonRFIDbodyUSB.BackColor = Color.Yellow;
                }
                else if ((USBTest == 7) && (gv.ENDCAP_USB_TEST != "PASS"))
                {
                    gv.ENDCAP_USB_SCAN = "YELO";
                    buttonRFIDendcapUSB.BackColor = Color.Yellow;
                }
                if (!gv.CUPHOLDER_USB_SCAN.Contains("GRAY")) USBTest = 2;
            }
        }

        private void PollInputs(out bool bConsole_PIP_55Way_Detected, out bool b5VUSBDetected)
        {
            if (OperatorInstructions.Text.Contains("138")) // MASTER RFID CARD #69 WAS DETECTED,  GOTO CSN # KEYPAD SCREEN
            {
                gv.BADGE_ACCESS = "Supervisor";
                gv.CSNEntryForm.textBoxVIN.Text = ManifestData.CSN;
                gv.CSNEntryForm.textBoxVIN.SelectAll();
                gv.CSNEntryForm.Show();
            }

            ReadDigitalInputs(out bConsole_PIP_55Way_Detected, out b5VUSBDetected);

            if (!b5VUSBDetected) b5vUSBwasDisconnected = true; 

            HandleManualRelease(bConsole_PIP_55Way_Detected);
        }

        public static byte ReadDigitalInputs(out bool b55WayDetected, out bool b5VUSBDetected)
        {
            byte[] data = new byte[] { 0, 0, 0, 0 };
            _ = AIOUSB.DIO_ReadAll(gv.diDIO, data);

            b55WayDetected = ((data[2] & 0x02) == 0x02); // Console 55-way detected
            bool bHeadliner2Detected = ((data[2] & 0x04) == 0x04); // Headliner 16-way X1 PIP Switch
            b5VUSBDetected = ((data[2] & 0x40) == 0x40);

            if (gv.J2HDebugMode) // debugging mode only!!! -- J2H DEBUG
            {
                b55WayDetected = gv.DiagnosticOverrideForm.DI1Detected;
                bHeadliner2Detected = gv.DiagnosticOverrideForm.DI1Detected;
                b5VUSBDetected = gv.DiagnosticOverrideForm.DI6Detected;
            }

            if (b55WayDetected) // Headliner_PIP_59Way has been detected
            {
                DrawDown59Way();
            }

            if (bHeadliner2Detected) // Headliner_PIP_55Way has been detected
            {
                DrawDown55Way();
            }
            return data[2];
        }

        private static void DrawDown59Way()
        {
            _ = AIOUSB.DIO_Write1(gv.diDIO, 11, 0); // Energize Sol #3   HEADLINER DRAW DOWN BH #1 59-way
        }

        private static void DrawDown55Way()
        {
            _ = AIOUSB.DIO_Write1(gv.diDIO, 12, 0); // Energize Sol #4   HEADLINER DRAW DOWN BH #2 55-way
        }

        public bool HandleManualRelease(bool bConsole1Detected)
        {
            // *****************************************************
            // ***** TEST FOR MANUAL RELEASE BUTTON PRESSED ON 55 WAY
            // *****************************************************
            if (bConsole1Detected) // Console_PIP_55-way
            {
                // DO NOTHING
                return false;
            }
            else  // MANUAL RELEASE PRESSED ON 59-WAY ?
            {
                if (ManifestData.b55_WayConnected)
                {
                    // RELEASE BOTH 55-WAY AND 16/16-WAY UNLATCH AND BLOW OFF SINCE IT HAS NO PUSH BUTTON
                    HeadlinerTestForm.Eject59Way();
                    HeadlinerTestForm.Eject55Way();
                    HeadlinerTestForm.Eject16WayX1AndX2();
                    ManifestData.b55_WayConnected = false;

                    if (gv.VSpy3ProcessHandle != null)
                        gv.VSpy3ProcessHandle.Kill();
                    gv.VSpy3ProcessHandle = null;
                    PLC.SendTestComplete();
                    return true;
                }
                return false;
            }
        } // HandleManualRelease

        //*************************************************************************************
        //***********  Prompt Operator to SCAN RFID Tag and test the 5VUSB ports
        //*************************************************************************************
        public void TestUSB_Endcap(bool b5vUSBConnected)
        {
            if (OperatorInstructions.Text.Contains("7"))
            {
                buttonRFIDendcapUSB.BackColor = Color.Aquamarine;
                if (gv.ENDCAP_USB_SCAN.Contains("YELO"))  // Manual or MOD CODE selected
                {
                    // ENDCAP RFID CARD SCAN DETECTED
                    //EndcapUSBrfidRESULT = 1; // Scanned
                    ManifestData.bEndcapUSBCardScanned = true;
                    TurnOnUSB5VLED();
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                    OperatorInstructions.Text += "TEST ENDCAP USB\n\r\n\r\n\r";
                    gv.ENDCAP_USB_SCAN = "PASS";
                    USBTest = 7;
                }
                else
                {
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                    OperatorInstructions.Text += "ONLY SCAN NEXT YELLOW\n\r\n\r\n\r";
                    OperatorInstructions.Text += "USB TEST CARD\n\r\n\r\n\r";
                }
            }

            if (gv.ENDCAP_USB_TEST != "PASS")
                if (gv.ENDCAP_USB_SCAN == "PASS")
                    if (b5vUSBwasDisconnected)
                        if (b5vUSBConnected)
                        {
                            buttonTESTendcapUSB.BackColor = Color.Aquamarine;
                            gv.ENDCAP_USB_TEST = "PASS";
                            button1.BackColor = Color.Aquamarine; // Connect USB Button Color on touch screen
                            TurnOffUSB5VLED();
                            b5vUSBwasDisconnected = false;
                            USBTest = 0;
                            OperatorInstructions.Text = ""; // CLEAR SCREEN
                            OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                            OperatorInstructions.Text += "SCAN NEXT YELLOW\n\r\n\r\n\r";
                            OperatorInstructions.Text += "USB TEST CARD\n\r\n\r\n\r";

                        }
        } // TestUSB_Endcap

        //*************************************************************************************
        //***********  Prompt Operator to SCAN RFID Tag and test the 5VUSB ports
        //*************************************************************************************
        public void TestUSB_Body(bool b5vUSBConnected)
        {
            if (OperatorInstructions.Text.Contains("4"))
            {
                buttonRFIDbodyUSB.BackColor = Color.Aquamarine;
                if (gv.BODY_USB_SCAN.Contains("YELO")) // Manual or MOD CODE selected
                {
                    // BODY RFID CARD SCAN DETECTED
                    //BodyUSBrfidRESULT = 1; // Scanned
                    ManifestData.bBodyUSBCardScanned = true;
                    TurnOnUSB5VLED();
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                    OperatorInstructions.Text += "TEST BODY USB\n\r\n\r\n\r";
                    gv.BODY_USB_SCAN = "PASS";
                    USBTest = 4;
                }
                else
                {
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                    OperatorInstructions.Text += "ONLY SCAN NEXT YELLOW\n\r\n\r\n\r";
                    OperatorInstructions.Text += "USB TEST CARD\n\r\n\r\n\r";
                }
            }

            if (gv.BODY_USB_TEST != "PASS")
                if (gv.BODY_USB_SCAN == "PASS")
                    if (b5vUSBwasDisconnected)
                        if (b5vUSBConnected)
                        {
                            TurnOffUSB5VLED();
                            buttonTESTbodyUSB.BackColor = Color.Aquamarine;
                            gv.BODY_USB_TEST = "PASS";
                            button1.BackColor = Color.Aquamarine; // Connect USB Button Color on touch screen
                            b5vUSBwasDisconnected = false;
                            USBTest = 0;
                            OperatorInstructions.Text = ""; // CLEAR SCREEN
                            OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                            OperatorInstructions.Text += "SCAN NEXT YELLOW\n\r\n\r\n\r";
                            OperatorInstructions.Text += "USB TEST CARD\n\r\n\r\n\r";
                        }
        } // TestUSB_Body

        //*************************************************************************************
        //***********  Prompt Operator to SCAN RFID Tag and test the 5VUSB ports
        //*************************************************************************************
        public void TestUSB_Cupholder(bool b5vUSBConnected)
        {
            if (OperatorInstructions.Text.Contains("2"))
            {
                buttonRFIDcupHolderUSB.BackColor = Color.Aquamarine;
                if (gv.CUPHOLDER_USB_SCAN.Contains("YELO"))  // Manual or MOD CODE selected
                {
                    // CUPHOLDER RFID CARD SCAN DETECTED
                    ManifestData.bCupholderUSBCardScanned = true;
                    gv.CUPHOLDER_USB_SCAN = "PASS";

                    TurnOnUSB5VLED();
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                    OperatorInstructions.Text += "TEST CUPHOLDER USB\n\r\n\r\n\r";
                    USBTest = 2;
                }
                else
                {
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                    OperatorInstructions.Text += "ONLY SCAN NEXT YELLOW\n\r\n\r\n\r";
                    OperatorInstructions.Text += "USB TEST CARD\n\r\n\r\n\r";
                }
            }

            if (gv.CUPHOLDER_USB_TEST != "PASS")
                if (gv.CUPHOLDER_USB_SCAN == "PASS")
                    if (b5vUSBwasDisconnected)
                        if (b5vUSBConnected)
                        {
                            TurnOffUSB5VLED();
                            buttonTESTcupHolderUSB.BackColor = Color.Aquamarine;
                            gv.CUPHOLDER_USB_TEST = "PASS";
                            button1.BackColor = Color.Aquamarine; // Connect USB Button Color on touch screen
                            b5vUSBwasDisconnected = false;
                            USBTest = 0;
                            OperatorInstructions.Text = ""; // CLEAR SCREEN
                            OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
                            OperatorInstructions.Text += "SCAN NEXT YELLOW\n\r\n\r\n\r";
                            OperatorInstructions.Text += "USB TEST CARD\n\r\n\r\n\r";
                        }
        } // TestUSB_Cupholder

        public static void TurnOnUSB5VLED()
        {
            _ = AIOUSB.DIO_Write1(gv.diDIO, 14, 0); // "ON" CONSOLE (LED) RFID TEST BLOCK  5VdcUSB & 12Vdc Power Outlet
        }

        public static void TurnOffUSB5VLED()
        {
            _ = AIOUSB.DIO_Write1(gv.diDIO, 14, 1); // "OFF" CONSOLE (LED) RFID TEST BLOCK  5VdcUSB & 12Vdc Powe5r Outlet
        }

        private void StartTest()
        {
            // **************************************************************************************************************************
            // ****************** Start the Console Test Automatic Sequence *************************************************************
            // **************************************************************************************************************************
            ManifestData.bStartTheConsoleTestAutomaticSequence = true;
            //***************************************************************************************************************************
            //***************************************************************************************************************************

            _ = AIOUSB.DIO_Write1(gv.diDIO, 11, 0); // Energize Sol #2   CONSOLE 59-way DRAW DOWN for bench validation at ETC
            _ = AIOUSB.DIO_Write1(gv.diDIO, 10, 0); // Energize Sol #2   CONSOLE 55-way DRAW DOWN
            labelDrawDownCON55.BackColor = Color.Aquamarine;
            labelPIPswitchCON55A.BackColor = Color.Aquamarine;
            labelPIPswitchCON55B.BackColor = Color.Aquamarine;

            // May require time delay or other way like analog channel reads to make sure the Drawdown and connector is fully seated before energizing the LATCH Solenoid ?
            _ = AIOUSB.DIO_Write1(gv.diDIO, 9, 0);  // Energize SOL #1   CONSOLE 55-way LATCH
            labelLatchCON55.BackColor = Color.Aquamarine;
            buttonConnect55Way.BackColor = Color.Aquamarine; // Connect 55-way Button Color on touch screen
            ManifestData.b55_WayPIPDetected = true;
            ManifestData.b55_WayDrawdownSolenoidEnergized = true;
            ManifestData.b55_WayLatchSolenoidEnergized = true;
            ManifestData.b55_WayConnected = true;

            OperatorInstructions.Text = ""; // CLEAR SCREEN
            OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
            OperatorInstructions.Text += "SCAN ANY YELLOW\n\r\n\r\n\r";
            OperatorInstructions.Text += "USB TEST CARD\n\r\n\r\n\r";

            PLC.SendTesterReady();
        } // StartTest

        private void RefreshInstructionText()
        {
            // Refresh Operator Startup Screen Prompt
            OperatorInstructions.Text = ""; // Clear Screen
            OperatorInstructions.Text += "<<< OPERATOR >>>\n\r\n\r\n\r";
            OperatorInstructions.Text += "FIRST CONNECT USB\n\r\n\r\n\r";
            OperatorInstructions.Text += "SECOND CONNECT 55-WAY\n\r\n\r";
        } // RefreshInstructionText

        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        public delegate bool TEnumWindowDelegate(IntPtr hWnd, IntPtr Context);

        [DllImport("user32.dll")]
        public static extern bool EnumChildWindows(IntPtr hWndParent, TEnumWindowDelegate lpEnumFunc, IntPtr lParam);

        private bool EnumWindowCallback(IntPtr hWnd, IntPtr Context)
        {
            _ = SetWindowPos(hWnd, 0, 0, 0, 0, 0, SWP_HIDEWINDOW | SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER | SWP_NOSIZE);
            return true;
        }

        public void EnsureTopmostFocused_bad()
        {
            // SetWindowPos(gv.ConsoleTestMainForm.Handle, 0, 0, 0, 0, 0, SWP_SHOWWINDOW);
            if (gv.VSpy3ProcessHandle != null)
            {
                _ = SetWindowPos(gv.VSpy3ProcessHandle.MainWindowHandle, 0, 0, 0, 0, 0, SWP_HIDEWINDOW | SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER | SWP_NOSIZE);
                TEnumWindowDelegate EnumWindowDelegate = new TEnumWindowDelegate(EnumWindowCallback);
                _ = EnumChildWindows(gv.VSpy3ProcessHandle.MainWindowHandle, EnumWindowDelegate, IntPtr.Zero);
                //_ = SetForegroundWindow(gv.VSpy3ProcessHandle.MainWindowHandle);
                //_ = ShowWindow(gv.VSpy3ProcessHandle.MainWindowHandle, SW_SHOWMINNOACTIVE);
            }
            //_ = SetForegroundWindow(GetDesktopWindow());
            _ = SetForegroundWindow(gv.ConsoleTestMainForm.Handle);
            _ = SetActiveWindow(gv.ConsoleTestMainForm.Handle);
            _ = OperatorInstructions.Focus();
        }




        public void EnsureTopmostFocused()
        {
            // SetWindowPos(gv.ConsoleTestMainForm.Handle, 0, 0, 0, 0, 0, SWP_SHOWWINDOW);
            if (gv.VSpy3ProcessHandle != null)
            {
                //_ = SetWindowPos(gv.VSpy3ProcessHandle.MainWindowHandle, 0, 0, 0, 0, 0, SWP_HIDEWINDOW | SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER | SWP_NOSIZE);
                //_ = SetForegroundWindow(gv.VSpy3ProcessHandle.MainWindowHandle);
                _ = ShowWindow(gv.VSpy3ProcessHandle.MainWindowHandle, SW_SHOWMINNOACTIVE);
            }
            //_ = SetForegroundWindow(GetDesktopWindow());
            _ = SetForegroundWindow(gv.ConsoleTestMainForm.Handle);
            _ = SetActiveWindow(gv.ConsoleTestMainForm.Handle);
            _ = OperatorInstructions.Focus();
        }
        public void EnsureTopmostFocused_old()
        {
            // SetWindowPos(gv.ConsoleTestMainForm.Handle, 0, 0, 0, 0, 0, SWP_SHOWWINDOW);
            if (gv.VSpy3ProcessHandle != null)
            {
                _ = SetForegroundWindow(gv.VSpy3ProcessHandle.MainWindowHandle);
                _ = ShowWindow(gv.VSpy3ProcessHandle.MainWindowHandle, 7);
            }
            //_ = SetForegroundWindow(GetDesktopWindow());
            _ = SetForegroundWindow(gv.ConsoleTestMainForm.Handle);
            _ = SetActiveWindow(gv.ConsoleTestMainForm.Handle);
            _ = OperatorInstructions.Focus();
        }

        /// <summary>
        /// Delegate to ensure FORM3 (gv.frm3) is visible, active, focused, and frm3.textBox1 has focus
        /// </summary>
        public void EnsureTopmostFocused_older()
        {
            // SetWindowPos(gv.ConsoleTestMainForm.Handle, 0, 0, 0, 0, 0, 0x0040);
            if (gv.VSpy3ProcessHandle != null)
                _ = ShowWindow(gv.VSpy3ProcessHandle.MainWindowHandle, 7); //SW_SHOWMINNOACTIVE
            _ = SetForegroundWindow(gv.ConsoleTestMainForm.Handle);
            _ = SetActiveWindow(gv.ConsoleTestMainForm.Handle);
            _ = OperatorInstructions.Focus();
        }

        public void UpdateProgressBar(int value)
        {
            /*
            if (gv.ConsoleTestMainForm.InvokeRequired)
            {
                _ = gv.ConsoleTestMainForm.Invoke(new Action<string>(AppendTextBox), new object[] { value });
                return;
            }
            */
            gv.ConsoleTestMainForm.progressBar1.Value = value;
        }

        /// <summary>
        /// launch VSpy3, wait for it to be ready, send "start" via TCP/IP, wait for it to be done, send "stop", wait for files to get written to disk
        /// </summary>
        private bool vSpyGatherPartNumbers_StateMachine()
        {
            switch (progressBar1.Value)
            {
                case 0:
                    UpdateProgressBar(1);
                    LaunchVSpy3();
                    _ = gv.ConsoleTestMainForm.Invoke(myDelegate);
                    break;

                case 1:
                    UpdateProgressBar(2);
                    _ = gv.ConsoleTestMainForm.Invoke(myDelegate);
                    break;

                case 2:
                    IPHostEntry IPHost = Dns.Resolve("127.0.0.1");
                    IPAddress[] addr = IPHost.AddressList;
                    EndPoint ep = new IPEndPoint(addr[0], 8255);
                    sock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                    _ = gv.ConsoleTestMainForm.Invoke(myDelegate);
                    try
                    {
                        sock.Connect(ep);
                        UpdateProgressBar(5);
                    }
                    catch (Exception)
                    {
                        Trace.WriteLine("Exception in sock.Connect");
                        UpdateProgressBar(6);
                        //throw;//handle TCP socket errors here
                    }
                    break;

                case 6: //handle socket exception
                /* fall through until exception handler */
                case 5:
                    gv.ConsoleTestMainForm.TopMost = false;
                    StartVSpyViaTCP();
                    _ = gv.ConsoleTestMainForm.Invoke(myDelegate);
                    UpdateProgressBar(7);
                    break;
                /*
                                case 7..29:
                                    // wait for it to be done
                                    break;
                */
                case 30:
                    StopVSpyViaTCP();
                    UpdateProgressBar(progressBar1.Value + 1);
                    _ = gv.ConsoleTestMainForm.Invoke(myDelegate);
                    break;

                case 31:
                    if (sock.Connected) sock.Shutdown(SocketShutdown.Both);
                    UpdateProgressBar(progressBar1.Value + 1);
                    break;

                case 32:
                    if (sock.Connected) sock.Close();
                    UpdateProgressBar(progressBar1.Value + 1);
                    break;
                /*
                            case 33..43:
                                // wait for it to be done
                                break;
                */
                case 44:
                    Trace.WriteLine("Used Text API to send START for Spy3 macro");
                    UpdateProgressBar(45);
                    break;

                case 45:// idle state sits here until progressBar1.Value is set back to zero
                    return true;
                //break;

                /* Default case just updates the progress bar and proceeds after the next timer tick
                 * note: timer is now 500ms
                 */
                default:
                    _ = gv.ConsoleTestMainForm.Invoke(myDelegate);
                    Trace.WriteLine("unexpected or delay state reached in vSpyGatherPartNumbers: " + progressBar1.Value.ToString());
                    UpdateProgressBar(progressBar1.Value + 1);
                    break;
            }
            return false;
        }

        private void LaunchVSpy3()
        {
            Trace.WriteLine("BEGIN LaunchVSpy3");
            Trace.Indent();
            if (File.Exists(@"C:\Android\Data Directory\Default\SIBPNs.txt"))
                File.Delete(@"C:\Android\Data Directory\Default\SIBPNs.txt");

            if (File.Exists(@"C:\Android\Data Directory\Default\Trailer Switch.txt"))
                File.Delete(@"C:\Android\Data Directory\Default\Trailer Switch.txt");

            if (File.Exists(@"C:\Android\Data Directory\Default\WCMPNs.txt"))
                File.Delete(@"C:\Android\Data Directory\Default\WCMPNs.txt");

            if (File.Exists(@"C:\Android\Data Directory\Default\EOCMPNsCAN4.txt"))
                File.Delete(@"C:\Android\Data Directory\Default\EOCMPNsCAN4.txt");

            if (File.Exists(@"C:\Android\Data Directory\Default\EOCMPNs.txt"))
                File.Delete(@"C:\Android\Data Directory\Default\EOCMPNs.txt");

            if (File.Exists(@"C:\Android\Data Directory\Default\HeatedSeat.txt"))
                File.Delete(@"C:\Android\Data Directory\Default\HeatedSeat.txt");

            try
            {
                gv.ConsoleTestMainForm.TopMost = true;
                ProcessProperties.StartInfo.UseShellExecute = false;
                ProcessProperties.StartInfo.FileName = "c:\\Program Files (X86)\\Vehicle spy 3\\vspy3.exe";
                ProcessProperties.StartInfo.Arguments = "c:\\Android\\Data Directory\\Default\\Test71.vs3exe";
                ProcessProperties.StartInfo.WindowStyle = ProcessWindowStyle.Minimized; // change to .Hidden
                if (gv.VSpy3ProcessHandle == null)
                    gv.VSpy3ProcessHandle = Process.Start(ProcessProperties.StartInfo);
                if (gv.VSpy3ProcessHandle !=null)
                    _ = ShowWindow(gv.VSpy3ProcessHandle.MainWindowHandle, SW_SHOWMINNOACTIVE);
            }
            catch (Exception)
            {
                Trace.WriteLine("Exception in LaunchVSpy3");
                //throw;
            }
            Trace.Unindent();
            Trace.WriteLine("END LaunchVSpy3()");
        }

        private void StartVSpyViaTCP()
        {
            Trace.WriteLine("BEGIN StartVSpyViaTCP()");
            Trace.Indent();
            try
            {
                byte[] RecvBytes = new byte[256];
                string Get = "start\r\n"; // Abnormal program termination
                byte[] ByteGet = Encoding.ASCII.GetBytes(Get);
                _ = sock.Send(ByteGet, ByteGet.Length, 0);
                int bytes = sock.Receive(RecvBytes, RecvBytes.Length, 0);
                string strRetPage = null;
                strRetPage += Encoding.ASCII.GetString(RecvBytes, 0, bytes);
                Trace.WriteLine(strRetPage); // "ok start"
            }
            catch (Exception)
            {
                Trace.WriteLine("Exception in StartVSpyViaTCP");
                //throw;
            }
            Trace.Unindent();
            Trace.WriteLine("END StartVSpyViaTCP()");
        }

        private void StopVSpyViaTCP()
        {
            Trace.WriteLine("BEGIN StartVSpyViaTCP()");
            Trace.Indent();

            try
            {
                Trace.WriteLine("StopVSpyViaTCP BEGIN");
                byte[] RecvBytes = new byte[256];
                string GGet = "stop\r\n";
                byte[] BByteGet = Encoding.ASCII.GetBytes(GGet);
                _ = sock.Send(BByteGet, BByteGet.Length, 0);

                int bbytes = sock.Receive(RecvBytes, RecvBytes.Length, 0);
                string sstrRetPage = null;
                sstrRetPage += Encoding.ASCII.GetString(RecvBytes, 0, bbytes);
                Trace.WriteLine(sstrRetPage); // "ok stop"
            }
            catch (Exception)
            {
                Trace.WriteLine("exception in StopVSpyViaTCP");
                //throw;
            }
            Trace.Unindent();
            Trace.WriteLine("END StopVSpyViaTCP()");
        }

        // ********************************************************************************************************************
        // ****** Read Global Vriables from SHARP7 PLC MANIFEST READ TO SEQUENCE WHICH USB TO TEST ****************************
        // ********************************************************************************************************************
        // *******  CUPHOLDER USB ...... MOD CODE # 4301 ..... Broadcast String (13529875FW85) ********************************
        // *******  BODY USB............ MOD CODE # 3002 ..... Broadcast String (13529874FW86) ********************************
        // *******  BODY USB............ NO PART ............. Broadcast String (----****FT05) ********************************
        // *******  ENDCAP USB ......... MOD CODE # 1502 ..... Broadcast String (13525890FT12) ********************************
        // ********************************************************************************************************************
        // ********************************************************************************************************************

        // ***********************************************************************************************************
        // ************** Start of File Parsing Section  *************************************************************
        // ***********************************************************************************************************
        private bool ParseVSpyResultsFiles()
        {
            Trace.WriteLine("BEGIN ParseVSpyResultsFiles()");
            Trace.Indent();

            try
            {
                ReadWCM();
                ReadSIB();
                ReadEOCM();
                ReadSEAT();
                ReadTBS();
            }
            catch (Exception)
            {
                Trace.WriteLine("Exception while reading results files");
                //handle file read errors
                //throw;
            }
            try
            {
                ParseWCM();
                ParseSIB();
                ParseEOCM();
                ParseSeat();
                ParseTBS();
            }
            catch (Exception)
            {
                Trace.WriteLine("Exception while parsing results data");
                //handle parsing failures
                //throw;
            }

            // Trial writing test log to file
            // string LOGstrFullFIleName = "c:/Android/Data Directory/Default/TestLog.txt";
            // StreamWriter srFileWriter = new StreamWriter(LOGstrFullFIleName);
            // srFileWriter.WriteLine(textBoxRESULTS.Text);
            // srFileWriter.Close();

            GenerateConsoleTestResultFile();

            ManifestData.bParsedMultipleTXTFiles = true;

            Trace.Unindent();
            Trace.WriteLine("END ParseVSpyResultsFiles()");
            return true;
        } // ParseVSpyResultsFiles


        private static void GenerateConsoleTestResultFile()
        {
            Trace.WriteLine("BEGIN GenerateConsoleTestResultFile()");
            Trace.Indent();

            //*******************************************************************************
            // ******************************************************************************
            // ******** Trial writing CONSOLE test result file for Larry to TX using Sharp7
            //*******************************************************************************
            //*******************************************************************************

            //string CONSOLE_strFullFIleName = "c:/Android/Data Directory/Default/CONSOLE_TESTER_2.txt";
            string CONSOLE_strFullFIleName = "c:/Android/Data Directory/Default/ConTestLog." + DateTime.Now.ToString("yyyymmdd-Hmmss") + ".txt";

            StreamWriter CONSOLE_FileWriter = new StreamWriter(CONSOLE_strFullFIleName);

            CONSOLE_FileWriter.WriteLine("ETC Console 2,");  // Tester Name(13) ETC Console 2,
            CONSOLE_FileWriter.WriteLine("06/29/21,"); // Test Date(10)  MM / DD / YEAR,
            CONSOLE_FileWriter.WriteLine("07:19:00,"); // Cycle Start Time Stamp(8), Military HR.MN.SC,
            CONSOLE_FileWriter.WriteLine("07:19:45,"); // Cycle Complete Time Stamp(8), Military HR.MN.SC,
            CONSOLE_FileWriter.WriteLine("0045,"); // Total Cycle Time(seconds)(4), 9999,
            CONSOLE_FileWriter.WriteLine("0000000355149,"); // Build ID # (10),  0000000355149,
            CONSOLE_FileWriter.WriteLine("0000004364960,"); // CSN # (13), Sequence #	0000004364960,

            ReportWCMResults(CONSOLE_FileWriter);
            ReportSIBResults(CONSOLE_FileWriter);
            ReportEOCMResults(CONSOLE_FileWriter);

            CONSOLE_FileWriter.WriteLine("1234,"); // Test 1 Mod Code(4), 1234,
            CONSOLE_FileWriter.WriteLine("Test Channel 1 * *****,"); // Test 1 Channel Name(20),	Test Channel 1 * *****,
            CONSOLE_FileWriter.WriteLine("PASS,"); // Test 1 Result(4), PASS,
            CONSOLE_FileWriter.WriteLine("12.00,"); // Test 1 Data Volts DC(5), 	12.00,
            CONSOLE_FileWriter.WriteLine(">= 12.00,"); // Test 1 PASS Voltage Limit >= 12.00,
            CONSOLE_FileWriter.WriteLine("<= 12.00,"); // Test 1 FAIL Voltage Limit <= 12.00,
            CONSOLE_FileWriter.WriteLine("******************************,");  // Test 1 Result Comments(30),		******************************,
            CONSOLE_FileWriter.WriteLine("5678,");   // Test 2 Mod Code(4),	5678,
            CONSOLE_FileWriter.WriteLine("Test Channel 2 * *****,");  // Test 2 Channel Name(20),	Test Channel 2 * *****,
            CONSOLE_FileWriter.WriteLine("PASS,");   // Test 2 Result(4), PASS,
            CONSOLE_FileWriter.WriteLine("12.00,");   // Test 2 Data Volts DC(5), 12.00,
            CONSOLE_FileWriter.WriteLine(">= 12.00,");   // Test 2 PASS Voltage Limit >= 12.00,
            CONSOLE_FileWriter.WriteLine("<= 12.00,");   // Test 2 FAIL Voltage Limit <= 12.00,
            CONSOLE_FileWriter.WriteLine("******************************,");   // Test 2 Result Comments(30), 	******************************,
            CONSOLE_FileWriter.Close();

            Trace.Unindent();
            Trace.WriteLine("END GenerateConsoleTestResultFile()");

        }

        private static void ReportEOCMResults(StreamWriter CONSOLE_FileWriter)
        {
            Trace.WriteLine("ReportEOCMResults()");

            if (gv.EOCM.Contains("PASS"))  // Manual or MOD CODE selected
            {
                gv.EOCM_BASE_PN = gv.EOCM_BASE_PN.Trim();
                gv.EOCM_END_PN = gv.EOCM_END_PN.Trim();
                gv.EOCM_BOOT_PN = gv.EOCM_BOOT_PN.Trim();
                gv.EOCM_ASWID_PN = gv.EOCM_ASWID_PN.Trim();
                gv.EOCM_ADATAID_PN = gv.EOCM_ADATAID_PN.Trim();

                CONSOLE_FileWriter.WriteLine("3301,"); // EOCM MOD CODE
                CONSOLE_FileWriter.WriteLine(gv.EOCM_BASE_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.EOCM_END_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.EOCM_BOOT_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.EOCM_ASWID_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.EOCM_ADATAID_PN + ",");
            }
        }

        private static void ReportSIBResults(StreamWriter CONSOLE_FileWriter)
        {
            Trace.WriteLine("ReportSIBResults()");
            if (gv.SIB.Contains("PASS"))  // Manual or MOD CODE selected
            {
                gv.SIB_BASE_PN = gv.SIB_BASE_PN.Trim();
                gv.SIB_END_PN = gv.SIB_END_PN.Trim();
                gv.SIB_BOOT_PN = gv.SIB_BOOT_PN.Trim();
                gv.SIB_ASWID_PN = gv.SIB_ASWID_PN.Trim();
                gv.SIB_ADATAID_PN = gv.SIB_ADATAID_PN.Trim();

                CONSOLE_FileWriter.WriteLine("2701,"); // SIB MOD CODE
                CONSOLE_FileWriter.WriteLine(gv.SIB_BASE_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.SIB_END_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.SIB_BOOT_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.SIB_ASWID_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.SIB_ADATAID_PN + ",");
            }
        }

        private static void ReportWCMResults(StreamWriter CONSOLE_FileWriter)
        {
            Trace.WriteLine("ReportWCMResults()");
            if (gv.WCM.Contains("PASS"))  // Manual or MOD CODE selected
            {
                gv.WCM_BASE_PN = gv.WCM_BASE_PN.Trim();
                gv.WCM_END_PN = gv.WCM_END_PN.Trim();
                gv.WCM_BOOT_PN = gv.WCM_BOOT_PN.Trim();
                gv.WCM_ASWID_PN = gv.WCM_ASWID_PN.Trim();
                gv.WCM_ADATAID_PN = gv.WCM_ADATAID_PN.Trim();

                CONSOLE_FileWriter.WriteLine("2901,"); // MCN MOD CODE
                CONSOLE_FileWriter.WriteLine(gv.WCM_BASE_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.WCM_END_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.WCM_BOOT_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.WCM_ASWID_PN + ",");
                CONSOLE_FileWriter.WriteLine(gv.WCM_ADATAID_PN + ",");
            }
        }

        private void ParseTBS()
        {
            Trace.WriteLine("BEGIN ParseTBS()");
            Trace.Indent();

            if (gv.TBS.Contains("YELO"))  // Manual or MOD CODE selected
            {
                //****************************************************************************************
                //***** PARSE THE LIN TRAILER SWITCH TEST RESULTS FROM THE (Trailer Switch.txt) FILE
                //****************************************************************************************
                Trace.WriteLine("ParseTBS; Contains YELO");

                // now locate Resault of PN read (0 = No Read) (1 = Able to read all 5 Part Numbers)
                int LIN_TBS_Result = ManifestData.TBSLog.IndexOf("Trailer Switch LIN, ");
                string LIN_TBS_ResultS = ManifestData.TBSLog.Substring((LIN_TBS_Result), 32);

                if (ManifestData.bLINTrailerBrakeSwitchReadDetectionPassed)
                {
                    // Do Nothing
                }
                else // One Shot
                {
                    if (ManifestData.TBSLog.Contains("Trailer Switch LIN, Payload 31, 0, Payload 32, 0")) // Ed Needs to add results or Time and date so I know it is same test as PN checks
                    {
                        gv.TBS = "FAIL";
                        button_LIN_TBS.BackColor = Color.Orange;
                        //LIN TRAILER BRAKE SWITCH READ DETECTION Failed.  No LIN TRAILER BRAKE SWITCH Detected.
                        Trace.WriteLine("ParseTBS FAIL");
                    }
                    else // One Shot
                    {
                        gv.TBS = "PASS";
                        button_LIN_TBS.BackColor = Color.Aquamarine;
                        ManifestData.bLINTrailerBrakeSwitchReadDetectionPassed = true;
                        Trace.WriteLine("ParseTBS pass");
                        gv.LIN_TBS = "PASS";
                    }
                }
            }
            Trace.Unindent();
            Trace.WriteLine("END ParseTBS()");
        }

        private void ParseSeat()
        {
            Trace.WriteLine("BEGIN ParseSeat()");
            Trace.Indent();
            if (gv.SEAT.Contains("YELO"))  // Manual or MOD CODE selected
            {
                //****************************************************************************************
                //***** PARSE THE LIN HEATED SEAT SWITCH TEST RESULTS FROM THE (HeatedSeatSwitch.txt) FILE
                //****************************************************************************************

                // now locate Resault of PN read (0 = No Read) (1 = Able to read all 5 Part Numbers)
                int LIN_HSS_Result = ManifestData.SeatLog.IndexOf("Heated Seat LIN, ");
                string LIN_HSS_ResultS = ManifestData.SeatLog.Substring((LIN_HSS_Result), 32);

                if (ManifestData.bLINHeatedSeatSwitchReadDetectionPassed)
                {
                    // Do Nothing
                }
                else // One Shot
                {
                    if (ManifestData.SeatLog.Contains("Heated Seat LIN, Payload 15, 0, Payload 35,0")) // Ed Needs to add results or Time and date so I know it is same test as PN checks
                    {
                        gv.SEAT = "FAIL";
                        button_LIN_Heated_Seat_Switch.BackColor = Color.Orange;
                        //LIN HEATED SEAT SWITCH READ DETECTION Failed.  No LIN Heated Seat Sw. Detected.
                        Trace.WriteLine("Parse Seat FAIL");
                    }
                    else // One Shot
                    {
                        gv.SEAT = "PASS";
                        button_LIN_Heated_Seat_Switch.BackColor = Color.Aquamarine;
                        ManifestData.bLINHeatedSeatSwitchReadDetectionPassed = true;
                        Trace.WriteLine("Parse Seat pass");
                        gv.LIN_SEAT_SW = "PASS";
                    }
                }
            }
            Trace.Unindent();
            Trace.WriteLine("END ParseSeat()");
        }

        private void ParseEOCM()
        {
            Trace.WriteLine("BEGIN ParseEOCM()");
            Trace.Indent();
            if (gv.EOCM.Contains("YELO"))  // Manual or MOD CODE selected
            {
                //*********************************************************************
                //***** PARSE THE EOCM PART NUMBERS FROM THE (EOCMPNsCAN4.txt) FILE
                //*********************************************************************

                // now locate Resault of PN read (0 = No Read) (1 = Able to read all 5 Part Numbers)
                int EOCM_Result = ManifestData.EOCMLog.IndexOf("EOCM Part Numbers, Test restut: ");
                string EOCM_ResultS = ManifestData.EOCMLog.Substring((EOCM_Result), 33);

                if (ManifestData.bEOCMModulePartNumberReadPassed)
                {
                    // Do Nothing
                }
                else // Parse out the Module Part Numbers
                {
                    if (EOCM_ResultS.Contains("1")) // 1 = Able to read all 5 Part Numbers
                    {
                        TextBox_Date_of_PN_Read.Text = ManifestData.EOCMLog.Substring((EOCM_Result + 34), 10);
                        TextBox_Time_of_PN_Read.Text = ManifestData.EOCMLog.Substring((EOCM_Result + 45), 14);
                        textBoxEOCM_BaseModelPartNumber8.Text = ManifestData.EOCMLog.Substring((EOCM_Result + 62), 8);
                        textBoxEOCM_AppDataID10.Text = ManifestData.EOCMLog.Substring((EOCM_Result + 71), 10);
                        textBoxEOCM_AppSWID10.Text = ManifestData.EOCMLog.Substring((EOCM_Result + 84), 11);
                        textBoxEOCM_BootSWID8.Text = ManifestData.EOCMLog.Substring((EOCM_Result + 95), 9);
                        textBoxEOCM_EndModelPartNumber8.Text = ManifestData.EOCMLog.Substring((EOCM_Result + 105), 10);

                        buttonEOCM.BackColor = Color.Aquamarine;

                        ManifestData.bEOCMModulePartNumberReadPassed = true;

                        gv.EOCM = "PASS";
                        gv.EOCM_BASE_PN = textBoxEOCM_BaseModelPartNumber8.Text;
                        gv.EOCM_END_PN = textBoxEOCM_EndModelPartNumber8.Text;
                        gv.EOCM_BOOT_PN = textBoxEOCM_BootSWID8.Text;
                        gv.EOCM_ASWID_PN = textBoxEOCM_AppSWID10.Text;
                        gv.EOCM_ADATAID_PN = textBoxEOCM_AppDataID10.Text;
                        gv.PN_TIME_READ = TextBox_Time_of_PN_Read.Text;
                        gv.PN_DATE_READ = TextBox_Date_of_PN_Read.Text;
                        Trace.WriteLine("Parse EOCM pass");
                    }
                    else // 0 = No Read
                    {
                        textBoxEOCM_AppDataID10.Text = " NO READ ";
                        textBoxEOCM_AppSWID10.Text = " NO READ ";
                        textBoxEOCM_BootSWID8.Text = " NO READ ";
                        textBoxEOCM_EndModelPartNumber8.Text = " NO READ ";
                        textBoxEOCM_BaseModelPartNumber8.Text = " NO READ ";

                        //EOCM Module Part Number Read Failed.  No EOCM Module Detected.
                        Trace.WriteLine("Parse EOCM FAIL");
                        buttonEOCM.BackColor = Color.Orange;
                    }
                }
            }
            Trace.Unindent();
            Trace.WriteLine("END ParseEOCM()");
        }

        private void ParseSIB()
        {
            Trace.WriteLine("BEGIN ParseSIB()");
            Trace.Indent();
            if (gv.SIB.Contains("YELO"))  // Manual or MOD CODE selected
            {
                //*********************************************************************
                //***** PARSE THE SIB PART NUMBERS FROM THE (SIBPNs.txt) FILE
                //*********************************************************************

                // now locate Resault of PN read (0 = No Read) (1 = Able to read all 5 Part Numbers)
                int SIB_Result = ManifestData.SIBLog.IndexOf("SIB Part Numbers, Test restut: ");
                string SIB_ResultS = ManifestData.SIBLog.Substring((SIB_Result), 32);

                if (ManifestData.bSIBModulePartNumberReadPassed)
                {
                    // Do Nothing
                }
                else // Parse out the Module Part Numbers
                {
                    if (SIB_ResultS.Contains("1")) // 1 = Able to read all 5 Part Numbers
                    {
                        TextBox_Date_of_PN_Read.Text = ManifestData.SIBLog.Substring((SIB_Result + 34), 10);
                        TextBox_Time_of_PN_Read.Text = ManifestData.SIBLog.Substring((SIB_Result + 44), 15);
                        textBoxSIB_AppDataID10.Text = ManifestData.SIBLog.Substring((SIB_Result + 61), 10);
                        textBoxSIB_AppSWID10.Text = ManifestData.SIBLog.Substring((SIB_Result + 73), 10);
                        textBoxSIB_BootSWID8.Text = ManifestData.SIBLog.Substring((SIB_Result + 84), 9);
                        textBoxSIB_EndModelPartNumber8.Text = ManifestData.SIBLog.Substring((SIB_Result + 94), 9);
                        textBoxSIB_BaseModelPartNumber8.Text = ManifestData.SIBLog.Substring((SIB_Result + 104), 10);

                        buttonSIB.BackColor = Color.Aquamarine;

                        ManifestData.bSIBModulePartNumberReadPassed = true;
                        Trace.WriteLine("Parse SIB pass");
                        gv.SIB = "PASS";
                        gv.SIB_BASE_PN = textBoxSIB_BaseModelPartNumber8.Text;
                        gv.SIB_END_PN = textBoxSIB_EndModelPartNumber8.Text;
                        gv.SIB_BOOT_PN = textBoxSIB_BootSWID8.Text;
                        gv.SIB_ASWID_PN = textBoxSIB_AppSWID10.Text;
                        gv.SIB_ADATAID_PN = textBoxSIB_AppDataID10.Text;
                        gv.PN_TIME_READ = TextBox_Time_of_PN_Read.Text;
                        gv.PN_DATE_READ = TextBox_Date_of_PN_Read.Text;
                    }
                    else // 0 = No Read
                    {
                        //SIB Module Part Number Read Failed. No SIB Module Detected.
                        Trace.WriteLine("Parse SIB FAIL");
                        textBoxSIB_AppDataID10.Text = " NO READ ";
                        textBoxSIB_AppSWID10.Text = " NO READ ";
                        textBoxSIB_BootSWID8.Text = " NO READ ";
                        textBoxSIB_EndModelPartNumber8.Text = " NO READ ";
                        textBoxSIB_BaseModelPartNumber8.Text = " NO READ ";

                        buttonSIB.BackColor = Color.Orange;
                    }
                }
            }
            Trace.Unindent();
            Trace.WriteLine("END ParseEOCM()");
        }

        private void ParseWCM()
        {
            Trace.WriteLine("BEGIN ParseEOCM()");
            Trace.Indent()
                ;
            //if (global_variables.WCM.Contains("YELO"))  // Manual or MOD CODE selected

            //*********************************************************************
            //***** PARSE THE WCM PART NUMBERS FROM THE (WCMPNs.txt) FILE
            //*********************************************************************

            // now locate Resault of PN read (0 = No Read) (1 = Able to read all 5 Part Numbers)
            int WCM_Result = ManifestData.WCMLog.IndexOf("WCM Part Numbers, Test restut: ");
            string WCM_ResultS = ManifestData.WCMLog.Substring(WCM_Result, 32);

            if (ManifestData.bWCMModulePartNumberReadPassed)
            {
                // Do Nothing
            }
            else // Parse out the Module Part Numbers
            {
                if (WCM_ResultS.Contains("1")) // 1 = Able to read all 5 Part Numbers
                {
                    TextBox_Date_of_PN_Read.Text = ManifestData.WCMLog.Substring((WCM_Result + 34), 10);
                    TextBox_Time_of_PN_Read.Text = ManifestData.WCMLog.Substring((WCM_Result + 44), 15);
                    textBoxWCM_AppDataID10.Text = ManifestData.WCMLog.Substring((WCM_Result + 61), 10);
                    textBoxWCM_AppSWID10.Text = ManifestData.WCMLog.Substring((WCM_Result + 73), 10);
                    textBoxWCM_BootSWID8.Text = ManifestData.WCMLog.Substring((WCM_Result + 84), 9);
                    textBoxWCM_EndModelPartNumber8.Text = ManifestData.WCMLog.Substring((WCM_Result + 94), 9);
                    textBoxWCM_BaseModelPartNumber8.Text = ManifestData.WCMLog.Substring((WCM_Result + 104), 10);

                    buttonWCM.BackColor = Color.Aquamarine;
                    ManifestData.bWCMModulePartNumberReadPassed = true;
                    gv.WCM = "PASS";
                    gv.WCM_BASE_PN = textBoxWCM_BaseModelPartNumber8.Text;
                    gv.WCM_END_PN = textBoxWCM_EndModelPartNumber8.Text;
                    gv.WCM_BOOT_PN = textBoxWCM_BootSWID8.Text;
                    gv.WCM_ASWID_PN = textBoxWCM_AppSWID10.Text;
                    gv.WCM_ADATAID_PN = textBoxWCM_AppDataID10.Text;
                    gv.PN_TIME_READ = TextBox_Time_of_PN_Read.Text;
                    gv.PN_DATE_READ = TextBox_Date_of_PN_Read.Text;
                    Trace.WriteLine("Parse WCM pass");
                }
                else // 0 = No Read
                {
                    //WCM Module Part Number Read Failed. No WCM Module Detected.

                    textBoxWCM_AppDataID10.Text = " NO READ ";
                    textBoxWCM_AppSWID10.Text = " NO READ ";
                    textBoxWCM_BootSWID8.Text = " NO READ ";
                    textBoxWCM_EndModelPartNumber8.Text = " NO READ ";
                    textBoxWCM_BaseModelPartNumber8.Text = " NO READ ";
                    Trace.WriteLine("Parse WCM FAIL");
                    buttonWCM.BackColor = Color.Orange;
                }
            }
            Trace.Unindent();
            Trace.WriteLine("END ParseWCM()");
        }

        private void ReadTBS()
        {
            Trace.WriteLine("BEGIN ReadTBS()");
            Trace.Indent();
            if (gv.TBS.Contains("YELO"))  // Manual or MOD CODE selected
            {
                //******************************************************
                //******* TBS - LIN Trailer Brake Switch Detection File
                //******************************************************
                string TBSstrFullFIleName = "c:/Android/Data Directory/Default/Trailer Switch.txt";
                StreamReader TBSsrFileReader = new StreamReader(TBSstrFullFIleName);
                ManifestData.TBSLog = TBSsrFileReader.ReadToEnd();
                Trace.WriteLine(ManifestData.TBSLog);
                TBSsrFileReader.Dispose();
                //srFileReader.Close();
            }
            Trace.Unindent();
            Trace.WriteLine("ReadTBS END");
        }

        private void ReadSEAT()
        {
            Trace.WriteLine("BEGIN ReadSEAT()");
            if (gv.SEAT.Contains("YELO"))  // Manual or MOD CODE selected
            {
                //*****************************************************
                //******* SEAT - LIN Heated Seat Switch Detection File
                //*****************************************************
                string SEATstrFullFIleName = "c:/Android/Data Directory/Default/HeatedSeat.txt";
                StreamReader SEATsrFileReader = new StreamReader(SEATstrFullFIleName);
                ManifestData.SeatLog = SEATsrFileReader.ReadToEnd();
                SEATsrFileReader.Dispose();
                //srFileReader.Close();
            }
            Trace.WriteLine("END ReadSEAT()");
        }

        private void ReadEOCM()
        {
            Trace.WriteLine("BEGIN ReadEOCM()");
            if (gv.EOCM.Contains("YELO"))  // Manual or MOD CODE selected
            {
                //***********************************
                //******* EOCM Part Number File
                //***********************************
                string EOCMstrFullFIleName = "c:/Android/Data Directory/Default/EOCMPNsCAN4.txt";
                StreamReader EOCMsrFileReader = new StreamReader(EOCMstrFullFIleName);
                ManifestData.EOCMLog = EOCMsrFileReader.ReadToEnd();
                Trace.WriteLine(ManifestData.EOCMLog);
                EOCMsrFileReader.Dispose();
                //srFileReader.Close();
            }
            Trace.WriteLine("END ReadEOCM()");
        }

        private void ReadSIB()
        {
            Trace.WriteLine("BEGIN ReadSIB()");
            if (gv.SIB.Contains("YELO"))  // Manual or MOD CODE selected
            {
                //***********************************
                //******* SIB Part Number File
                //***********************************
                string SIBstrFullFIleName = "c:/Android/Data Directory/Default/SIBPNs.txt";
                StreamReader SIBsrFileReader = new StreamReader(SIBstrFullFIleName);
                ManifestData.SIBLog = SIBsrFileReader.ReadToEnd();
                Trace.WriteLine(ManifestData.SIBLog);
                SIBsrFileReader.Dispose();
                // srFileReader.Close();
            }
            Trace.WriteLine("END ReadSIB()");
        }

        private void ReadWCM()
        {
            Trace.WriteLine("BEGIN ReadWCM()");
            if (gv.WCM.Contains("YELO"))  // Manual or MOD CODE selected
            {
                Trace.WriteLine("ReadWCM; Contains YELO");
                //***********************************
                //******* WCM Part Number File
                //***********************************
                string WCMstrFullFIleName = "c:/Android/Data Directory/Default/WCMPNs.txt";
                StreamReader srFileReader = new StreamReader(WCMstrFullFIleName);
                ManifestData.WCMLog = srFileReader.ReadToEnd();
                Trace.WriteLine(ManifestData.WCMLog);
                srFileReader.Dispose();
                // srFileReader.Close();
            }
            Trace.WriteLine("END ReadWCM()");
        }

        private void buttonViewResults_Click(object sender, EventArgs e)  // FOR DEMO ONLY!  VIEW RESULTS
        {
            Trace.WriteLine("buttonViewResults_Click()");
            Timer1.Stop();
            gv.DiagnosticForm.Show();
        }

        private void ShowDiagnosticForm_Click(object sender, EventArgs e) // FOR DEMO ONLY!  START ANALOG TEST
        {
            Trace.WriteLine("ShowDiagnosticForm_Click()");
            Timer1.Stop();
            gv.DiagnosticForm.Show();
        }

        private void buttonOperatorAbort_Click(object sender, EventArgs e)// OPERATOR ABORT TEST CYCLE
        {
            Trace.WriteLine("buttonOperatorAbort_Click()");
            Timer1.Stop();
            PLC.SendTestFailed();
            PLC.SendTestComplete();
            gv.ConsoleTestResultForm.Show();
        }

        private void buttonExit_Click(object sender, EventArgs e)
        {
            Trace.WriteLine("buttonExit_Click()");
            Timer1.Stop();
            PLC.SendTestComplete();
            gv.bCloseProgram = true; // TODO: only set to true if developer means it!!
            if (gv.VSpy3ProcessHandle != null)
                _ = gv.VSpy3ProcessHandle.CloseMainWindow();
            Close();
        }

        private void ConsoleConnectTestForm3_FormClosing(object sender, FormClosingEventArgs e)
        {
            Trace.WriteLine("ConsoleConnectTestForm3_FormClosing()");
            if (gv.VSpy3ProcessHandle != null)
                if (!gv.VSpy3ProcessHandle.HasExited)
                {
                    Trace.WriteLine("killing VSpy3()");
                    gv.VSpy3ProcessHandle.Kill();
                }
            gv.VSpy3ProcessHandle = null;
        }

        private void buttonDemoManifest_Click(object sender, EventArgs e)
        {
            Trace.WriteLine("buttonDemoManifest_Click()");
            //*************************************************************
            // ***** PREMIUM MANIFEST FOR RED RABBIT TEST NOT FROM THE PLC
            //****** MUST REMOVE REMARKS FOR PRODUCTION
            //*************************************************************

            // CONSOLE MANIFEST - Highline (default)                                                          
            // remove this comment to update PLC with manual manifest
            PLCInterface.WriteManifest(Encoding.ASCII.GetBytes("0000000355149,103xxxxxxxxxx,0000004364960,13525890FT12,1502,13529874FW86,3002,13529875FW85,4301,135619101FT06,402,13592451FT45,1302,85117464FT09,1101,84638621GO2A,2701,13592452FW74,3501,84952174FW83,4201,84786104FW68,3101,84960563M33V,3301,13537143M08L,2901,"));
        }
    }// ********************************************************************************************
} // *****************************************************