﻿using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface
using System;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;

namespace Sample1
{
    public partial class DiagnosticForm : Form
    {
        public double[] ADC_CH = new double[128];
        private const int ERROR_SUCCESS = 0; // Win32 standard response code
        private const byte bmDifferential = 0x8; // const for Range Selection to indicate "Differential"
        private uint ConfigBufSize = 21; // sizeof(Config);

        public DiagnosticForm()
        {
            Trace.WriteLine("DiagnosticForm() constructor");
            Trace.Indent();

            Trace.WriteLine("Tester Name is "+PLC.Results.TesterName);
            InitializeComponent();

            if (PLC.Results.TesterName.Contains("CONSOLE"))
            {                // CONSOLE TEST SELECTED
                labelCONSOLE_0_15.Visible = true;
                labelCONSOLE_16_31.Visible = true;
                labelHEADLINER_0_15.Visible = false;
                labelHEADLINER_16_31.Visible = false;

                CH0.Text = "101 Battery Feed EOCM";
                CH1.Text = "102 Battry Feed WCM / Aux. A/V";
                CH2.Text = "103 SIB Battery Feed";
                CH3.Text = "104 Battery Feed Row 2 Heated Seat Switch";
                CH4.Text = "105 Ground";
                CH5.Text = "106 Ground";
                CH6.Text = "107 NOT USED";
                CH7.Text = "108 NOT USED";

                CH8.Text = "109 SIB Ignition";
                CH9.Text = "110 EBCM LIN (2)";
                CH10.Text = "111 PEPS Antenna (+)";
                CH11.Text = "112 PEPS Antenna (-)";
                CH12.Text = "113 AC Power Outlet Sense HI";
                CH13.Text = "114 Ground AC Outlet Only";
                CH14.Text = "115 DC to AC Inverter Control";
                CH15.Text = "116 Immobilizer Antenna (+)";

                CH16.Text = "117 Immobilizer Antenna (-)";
                CH17.Text = "118 Secondary Fused Battery (+) 23";
                CH18.Text = "119 LIN (11) Signal Rear Heated Seat Switch";
                CH19.Text = "120 Powertrain Sensor Bus Enable";
                CH20.Text = "121 Ignition Feed Console Lamp";
                CH21.Text = "122 LED Backlight Dimming Control";
                CH22.Text = "123 Battery Feed Row2 Aux. USB Power";
                CH23.Text = "124 Ground Audio Jack Only";

                CH24.Text = "125 NOT USED";
                CH25.Text = "126 NOT USED";
                CH26.Text = "127 NOT USED";
                CH27.Text = "128 12Vdc Power Outlet";
                CH28.Text = "129 5Vdc USb Type-A / RETEST Input 12V";
                CH29.Text = "130 ABORT Input 12V";
                CH30.Text = "131 FAIL Input 12V";
                CH31.Text = "132 PASS Input 12V";
            }
            else
            {
                // HEADLINER TEST SELECTED
                labelCONSOLE_0_15.Visible = false;
                labelCONSOLE_16_31.Visible = false;
                labelHEADLINER_0_15.Visible = true;
                labelHEADLINER_16_31.Visible = true;
            }

            if (gv.DiagnosticOverrideForm != null)
            {
                gv.DiagnosticOverrideForm.NameADC(0, CH0.Text);
                gv.DiagnosticOverrideForm.NameADC(1, CH1.Text);
                gv.DiagnosticOverrideForm.NameADC(2, CH2.Text);
                gv.DiagnosticOverrideForm.NameADC(3, CH3.Text);
                gv.DiagnosticOverrideForm.NameADC(4, CH4.Text);
                gv.DiagnosticOverrideForm.NameADC(5, CH5.Text);
                gv.DiagnosticOverrideForm.NameADC(6, CH6.Text);
                gv.DiagnosticOverrideForm.NameADC(7, CH7.Text);
                gv.DiagnosticOverrideForm.NameADC(8, CH8.Text);
                gv.DiagnosticOverrideForm.NameADC(9, CH9.Text);
                gv.DiagnosticOverrideForm.NameADC(10, CH10.Text);
                gv.DiagnosticOverrideForm.NameADC(11, CH11.Text);
                gv.DiagnosticOverrideForm.NameADC(12, CH12.Text);
                gv.DiagnosticOverrideForm.NameADC(13, CH13.Text);
                gv.DiagnosticOverrideForm.NameADC(14, CH14.Text);
                gv.DiagnosticOverrideForm.NameADC(15, CH15.Text);
                gv.DiagnosticOverrideForm.NameADC(16, CH16.Text);
                gv.DiagnosticOverrideForm.NameADC(17, CH17.Text);
                gv.DiagnosticOverrideForm.NameADC(18, CH18.Text);
                gv.DiagnosticOverrideForm.NameADC(19, CH19.Text);
                gv.DiagnosticOverrideForm.NameADC(20, CH20.Text);
                gv.DiagnosticOverrideForm.NameADC(21, CH21.Text);
                gv.DiagnosticOverrideForm.NameADC(22, CH22.Text);
                gv.DiagnosticOverrideForm.NameADC(23, CH23.Text);
                gv.DiagnosticOverrideForm.NameADC(24, CH24.Text);
                gv.DiagnosticOverrideForm.NameADC(25, CH25.Text);
                gv.DiagnosticOverrideForm.NameADC(26, CH26.Text);
                gv.DiagnosticOverrideForm.NameADC(27, CH27.Text);
                gv.DiagnosticOverrideForm.NameADC(28, CH28.Text);
                gv.DiagnosticOverrideForm.NameADC(29, CH29.Text);
                gv.DiagnosticOverrideForm.NameADC(30, CH30.Text);
                gv.DiagnosticOverrideForm.NameADC(31, CH31.Text);
            }

            Trace.Unindent();
            Trace.WriteLine("END DiagnosticForm() constructor");
        }

        public void StartADCReadings()
        {
            Trace.WriteLine("BEGIN StartADCReadings()");
            Trace.Indent();
            InitializeADCReadings();
            ExecuteButton.Enabled = false;
            StopButton.Enabled = true;

            Trace.WriteLine("STARTING ScanTIMER");
            ScanTimer.Start();
            Trace.Unindent();
            Trace.WriteLine("END StartADCReadings");
        }

        public void InitializeADCReadings()
        {
            Trace.WriteLine("BEGIN InitializeADCReadings()");
            Trace.Indent();

            Trace.WriteLine("STOPPING ScanTimer");
            ScanTimer.Stop();

            // attempt to autocal but fall-back to no-calibration
            if (AIOUSB.ADC_SetCal(gv.diADC, ":AUTO:") != ERROR_SUCCESS)
                _ = AIOUSB.ADC_SetCal(gv.diADC, ":1TO1:");

            byte[] ConfigBuf = new byte[21];
            byte ADCRangeCode = 1; // ±10V input range: change here!

            for (int i = 0; i < 16; i++)
                ConfigBuf[i] = (byte)(ADCRangeCode); // can be different per channel

            ConfigBuf[16] = 0; // not calibrate mode
            ConfigBuf[17] = 5; // timer scan start
            ConfigBuf[18] = 0xF0; // channel 0 through F
            ConfigBuf[19] = 0xff; // max oversamples
            ConfigBuf[20] = 0x10; // set end-channel top-nybble

            var status = AIOUSB.ADC_SetConfig(gv.diADC, ConfigBuf, ref ConfigBufSize);
            if ((status != ERROR_SUCCESS) && !gv.J2HDebugMode)
            {
                // handle error
            }

            Trace.Unindent();
            Trace.WriteLine("END InitializeADCReadings()");
        }

        private void ExecuteButton_Click(object sender, EventArgs e)
        {
            Trace.WriteLine("BEGIN ExecuteButton_Click()");
            Trace.Indent();

            InitializeADCReadings();

            ExecuteButton.Enabled = false;
            StopButton.Enabled = true;

            Trace.WriteLine("STARTING ScanTimer");
            ScanTimer.Start();

            Trace.Unindent();
            Trace.WriteLine("END ExecuteButton_Click()");
        }

        private void ScanTimer_Tick(object sender, EventArgs e)
        {
            PLC.Heartbeat();
            AcquireAndEvaluateAnalogInputs();

            listBoxVoltage.Items.Clear();
            for (int CH = 0; CH < 32; ++CH)  // change 32 for the number of channels your device has
            {
                _ = listBoxVoltage.Items.Add(string.Format("Ch {0,2:D2}: {1,8:F4}", CH, this.ADC_CH[CH]));
            }

            if (PLC.Results.TesterName.Contains("CONSOLE"))
                ADCTestConsoleComplete();

            if (PLC.Results.TesterName.Contains("HEADLINER"))
                ADCTestHeadlinerComplete();
        }

        public void AcquireAndEvaluateAnalogInputs()
        {
            var status = AIOUSB.ADC_GetScanV(gv.diADC, ADC_CH);  // acquire scan of ADC data

            if ((status != ERROR_SUCCESS) && !gv.J2HDebugMode)
            {
                Trace.TraceError("ADC_GetScanV Error! Status#: " + status.ToString());
                ScanTimer.Stop();
                _ = MessageBox.Show("ADC_GetScanV Error! Status#: " + status.ToString());
            }
            else
            {

                /*for (int i = 0; i < 32; i++)
                {
                    Trace.Write(" " + ADC_CH[i].ToString("f3"));
                }
                Trace.WriteLine(".");*/


                if (PLC.Results.TesterName.Contains("HEADLINER"))
                {
                    ADCTestHeadliner1();
                    ADCTestHeadliner2();
                    ADCTestHeadliner3();
                    ADCTestHeadliner4();
                }

                if (PLC.Results.TesterName.Contains("CONSOLE"))
                {
                    ADCTestConsole1();
                    ADCTestConsole2();
                    ADCTestConsole3();
                }
            }
        }

        private void ADCTestHeadliner1()
        {

                //   if (ADC_CH[0] > 5) CH0.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Ground <<< *** GND
                //    if (ADC_CH[0] < 5) CH0.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Ground <<< *** GND

                //   if (ADC_CH[1] > 5) CH1.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Ground <<< *** GND
                //   if (ADC_CH[1] < 5) CH1.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Ground <<< *** GND

                //   if (ADC_CH[2] < 5) CH2.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Overhead Console Front Camera Battery Feed <<< *** +13.8Vdc
                //   if (ADC_CH[2] > 5) CH2.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Overhead Console Front Camera Battery Feed <<< *** +13.8Vdc

                //    if (ADC_CH[3] > 5) CH3.BackColor = Color.Orange; // FAIL (____ TEST)
                //   if (ADC_CH[3] > 5) CH3.BackColor = Color.Aquamarine;  // PASS (_____ TEST)

                //   if (ADC_CH[4] > 5) CH4.BackColor = Color.Orange; // FAIL (_____ TEST)
                //   if (ADC_CH[4] > 5) CH4.BackColor = Color.Aquamarine;  // PASS (_____ TEST)

                //   if (ADC_CH[5] > 5) CH5.BackColor = Color.Orange; // FAIL (___ TEST)
                //   if (ADC_CH[5] > 5) CH5.BackColor = Color.Aquamarine;  // PASS (___ TEST)

                //   if (ADC_CH[6] > 5) CH6.BackColor = Color.Orange; // FAIL (____ TEST)
                //   if (ADC_CH[6] > 5) CH6.BackColor = Color.Aquamarine;  // PASS (____ TEST)

                //   if (ADC_CH[7] > 5) CH7.BackColor = Color.Orange; // FAIL (____ TEST)
                //   if (ADC_CH[7] > 5) CH7.BackColor = Color.Aquamarine;  // PASS (____ TEST)

        }

        private void ADCTestHeadliner2()
        {
            //   if (ADC_CH[8] > 5) CH8.BackColor = Color.Orange;   // FAIL (HEADLINER TEST) Inside Rear View Mirror - Humidity Sensor IGN Feed
            //   if (ADC_CH[8] > 5) CH8.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST)  Inside Rear View Mirror - Humidity Sensor IGN Feed

            //    if (ADC_CH[9] > 5) CH9.BackColor = Color.Orange;   // FAIL (HEADLINER TEST) Cell Phone MIC (-)  <<<< **** GND
            //    if (ADC_CH[9] > 5) CH9.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Cell Phone MIC (-)  <<< **** GND

            if (gv.CMIC.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[10] = gv.DiagnosticOverrideForm.BADC10Valid ? 0 : 10;
                if (ADC_CH[10] > 1.3)
                {
                    CH10.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Cell Phone MIC (+) <<< *** 5Vdc 2.2K pull up resistor
                    gv.CMIC = "FAIL";
                    Trace.WriteLine("CMIC ADC_CH10 fail = " + ADC_CH[10].ToString());

                }

                if (ADC_CH[10] <= 1.3)  // PASS (HEADLINER TEST) Cell Phone MIC (+) <<< *** 2.2K pull up resistor
                {
                    CH10.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Cell Phone MIC (+) <<< *** 2.2K pull up resistor
                    gv.CMIC = "PASS";
                    Trace.WriteLine("CMIC ADC_CH10 pass = " + ADC_CH[10].ToString());
                }
            }

            //   if (ADC_CH[11] > 5) CH11.BackColor = Color.Orange; // FAIL (HEADLINER TEST) MIC (+) <<< *** 5Vdc 2.2K pull up resistor
            //   if (ADC_CH[11] > 5) CH11.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) MIC (+)  <<< 5Vdc 2.2K pull up resistor

            if (gv.MIC.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[11] = gv.DiagnosticOverrideForm.BADC11Valid ? 0 : 10;
                if (ADC_CH[11] >= 1.3)
                {
                    CH11.BackColor = Color.Orange; // FAIL (HEADLINER TEST) MIC (-) <<< *** GND
                    gv.MIC = "FAIL";
                    Trace.WriteLine("MIC ADC_CH11 fail = " + ADC_CH[11].ToString());
                }

                if (ADC_CH[11] < 1.3)
                {
                    CH11.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) MIC (-)  <<<  *** GND
                    gv.MIC = "PASS";
                    Trace.WriteLine("MIC ADC_CH11 pass = " + ADC_CH[11].ToString());
                }
            }

            //   if (ADC_CH[13] > 5) CH13.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Center High Mounted Stop Lamp Feed
            //   if (ADC_CH[13] > 5) CH13.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Center High Mounted Stop Lamp Feed

            //   if (ADC_CH[14] > 5) CH14.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Exterior Courtesy Lamp Feed
            //   if (ADC_CH[14] > 5) CH14.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Exterior Courtesy Lamp Feed

            //   if (ADC_CH[15] > 5) CH15.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Automatic Day Night Mirror Mirror (+)
            //   if (ADC_CH[15] > 5) CH15.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Automatic Day Night Mirror Mirror (+)
        }

        private void ADCTestHeadliner3()
        {
            //   if (ADC_CH[16] > 5) CH16.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Automatic Day Night Mirror (-)
            //   if (ADC_CH[16] > 5) CH16.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Automatic Day Night Mirror (-)

            //   if (ADC_CH[17] > 5) CH17.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Body Control Module LIN 5
            //   if (ADC_CH[17] > 5) CH17.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Body Control Module LIN 5

            //   if (ADC_CH[18] > 5) CH18.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Upfitter Accessory 5 Feed
            //   if (ADC_CH[18] > 5) CH18.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Upfitter Accessory 5 Feed

            if (gv.ANCMIC.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[19] = gv.DiagnosticOverrideForm.BADC19Valid ? 0 : 10;
                if (ADC_CH[19] > 1.5) // FAIL (HEADLINER TEST) Active Noise Canceling MIC
                {
                    CH19.BackColor = Color.Orange; // FAIL (HEADLINER TEST) MIC
                    gv.ANCMIC = "FAIL";
                    Trace.WriteLine("ANCMIC ADC_CH19 fail = " + ADC_CH[19].ToString());
                }

                if (ADC_CH[19] <= 1.5)  // PASS (HEADLINER TEST) Active Noise Canceling MIC
                {
                    CH19.BackColor = Color.Aquamarine; // FAIL (HEADLINER TEST) MIC
                    gv.ANCMIC = "PASS";
                    Trace.WriteLine("ANCMIC ADC_CH19 pass = " + ADC_CH[19].ToString());
                }
            }

            //   if (ADC_CH[20] > 5) CH20.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Active Noise Canceling MIC (-)
            //   if (ADC_CH[20] > 5) CH20.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Active Noise Canceling MIC (-)

            //   if (ADC_CH[21] > 5) CH21.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Row 2 Dome Lamp Switch
            //   if (ADC_CH[21] > 5) CH21.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Row 2 Dome Lamp Switch

            //  if (ADC_CH[22] > 5) CH22.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Row 2 Dome Lamp 2 Switch
            //  if (ADC_CH[22] > 5) CH22.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Row 2 Dome Lamp 2 Switch

            if (gv.LSLAMP.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[23] = gv.DiagnosticOverrideForm.BADC23Valid ? 0 : 10;
                if (ADC_CH[23] >= 5)
                {
                    CH23.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Left Sunshade Interior Lamp
                    gv.LSLAMP = "FAIL";
                    Trace.WriteLine("LSLAMP ADC_CH23 fail = " + ADC_CH[23].ToString());
                }

                if (ADC_CH[23] <= 5)
                {
                    CH23.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Left Sunshade Interior Lamp
                    gv.LSLAMP = "PASS";
                    Trace.WriteLine("LSLAMP ADC_CH23 pass = " + ADC_CH[23].ToString());
                }
            }
        }

        private void ADCTestHeadliner4()
        {
            if (gv.RSLAMP.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[24] = gv.DiagnosticOverrideForm.BADC24Valid ? 0 : 10;
                if (ADC_CH[24] >= 5)
                {
                    CH24.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Right Sunshade Interior Lamp
                    gv.RSLAMP = "FAIL";
                    Trace.WriteLine("RSLAMP ADC_CH24 fail = " + ADC_CH[24].ToString());
                }

                if (ADC_CH[24] < 5)
                {
                    CH24.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Right Sunshade Interior Lamp
                    gv.RSLAMP = "PASS";
                    Trace.WriteLine("RSLAMP ADC_CH24 pass = " + ADC_CH[24].ToString());
                }
            }

            //   if (ADC_CH[25] > 5) CH25.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Row 2 Dome Lamp Control
            //   if (ADC_CH[25] > 5) CH25.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Row 2 Dome Lamp Control

            if (gv.R2LAMPC.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[26] = gv.DiagnosticOverrideForm.BADC26Valid ? 0 : 10;
                if (ADC_CH[26] >= 3)
                {
                    CH26.BackColor = Color.Orange; // FAIL (HEADLINER TEST) Row 2 Dome Lamp 2 Control
                    gv.R2LAMPC = "FAIL";
                    Trace.WriteLine("R2LAMP ADC_CH26 fail = " + ADC_CH[26].ToString());
                }

                if (ADC_CH[26] < 3)
                {
                    CH26.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) Row 2 Dome Lamp 2 Control
                    gv.R2LAMPC = "PASS";
                    Trace.WriteLine("R2LAMP ADC_CH26 pass = " + ADC_CH[26].ToString());
                }
            }

            //   if (ADC_CH[27] < 5) CH27.BackColor = Color.White; // FAIL (NOT USED)
            //   if (ADC_CH[27] > 5) CH27.BackColor = Color.Aquamarine;  // PASS (NOT USED)

            //   if (ADC_CH[28] < 5) CH28.BackColor = Color.White; // FAIL (NOT USED)
            //   if (ADC_CH[28] > 5) CH28.BackColor = Color.Aquamarine;  // PASS (NOT USED)

            //   if (ADC_CH[29] > 5) CH29.BackColor = Color.Orange; // FAIL (NOT USED)
            //   if (ADC_CH[29] > 5) CH29.BackColor = Color.Aquamarine;  // PASS (NOT USED)

            //   if (ADC_CH[30] > 5) CH30.BackColor = Color.Orange; // FAIL (NOT USED)
            //    if (ADC_CH[30] > 5) CH30.BackColor = Color.Aquamarine;  // PASS (NOT USED)

            //   if (ADC_CH[31] > 5) CH31.BackColor = Color.Orange; // FAIL (NOT USED)
            //   if (ADC_CH[31] > 5) CH31.BackColor = Color.Aquamarine;  // PASS (NOT USED)
        }

        private void ADCTestHeadlinerComplete()
        {
            if (gv.MIC.Contains("PASS") || gv.CMIC.Contains("PASS") || gv.LSLAMP.Contains("PASS") || gv.RSLAMP.Contains("PASS") || gv.R2LAMPC.Contains("PASS"))
            {
                gv.RETEST = "YES";
                ScanTimer.Stop();
                //Form5 frm5 = new Form5();
                gv.HeadlinerTestResultsOptionsForm.Show();
            }
            else
            {
                gv.RETEST = "YES";
                ScanTimer.Stop();
                //Form5 frm5 = new Form5();
                gv.HeadlinerTestResultsOptionsForm.Show();
            }
        }

        private void ADCTestConsole1()
        {
            //   if (ADC_CH[8] > 5) CH8.BackColor = Color.Orange;   // FAIL (CONSOLE TEST) SIB Ignition
            //   if (ADC_CH[8] > 5) CH8.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) SIB Ignition

            //   if (ADC_CH[9] > 5) CH9.BackColor = Color.Orange;   // FAIL (CONSOLE TEST) EBCM LIN (2)
            //   if (ADC_CH[9] > 5) CH9.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) EBCM LIN (2)

            //   if (ADC_CH[11] > 5) CH11.BackColor = Color.Orange; // FAIL (CONSOLE TEST) PEPS Antenna (-)
            //   if (ADC_CH[11] > 5) CH11.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) PEPS Antenna (-)

            //   if (ADC_CH[13] > 5) CH13.BackColor = Color.Orange; // FAIL (CONSOLE TEST) Ground AC Outlet Only
            //   if (ADC_CH[13] > 5) CH13.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) Ground AC Outlet Only

            if (gv.HVPO.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[12] = gv.DiagnosticOverrideForm.BADC12Valid ? 10 : 0;
                if (ADC_CH[12] <= 0.6) // RED RABBIT TEST 1.2V CONNECTED ,  0VDC OPEN
                {
                    CH13.BackColor = Color.Orange; // FAIL (CONSOLE TEST) AC Power Outlet Sense HI
                    gv.HVPO = "FAIL";
                    Trace.WriteLine("HVPO ADC_CH12 fail = " + ADC_CH[12].ToString());
                }

                if (ADC_CH[12] > 0.6) // RED RABBIT TEST 1.2V CONNECTED ,  0VDC OPEN
                {
                    CH13.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) AC Power Outlet Sense HI
                    gv.HVPO = "PASS";
                    Trace.WriteLine("HVPO ADC_CH12 pass = " + ADC_CH[12].ToString());
                }
            }

            //   if (ADC_CH[14] > 5) CH14.BackColor = Color.Orange; // FAIL (CONSOLE TEST) DC to AC Inverter Control
            //   if (ADC_CH[14] > 5) CH14.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) DC to AC Inverter Control

            if (gv.IMOB.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[15] = gv.DiagnosticOverrideForm.BADC15Valid ? 0 : 10;
                if (ADC_CH[15] >= 1.2)
                {
                    CH15.BackColor = Color.Orange; // FAIL (CONSOLE TEST) Immobilizer Antenna (+)
                    gv.IMOB = "FAIL";
                    Trace.WriteLine("IMOB ADC_CH15 fail = " + ADC_CH[15].ToString());
                }

                if (ADC_CH[15] < 1.2)
                {
                    CH15.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) Immobilizer Antenna (+)
                    gv.IMOB = "PASS";
                    Trace.WriteLine("IMOB ADC_CH15 pass = " + ADC_CH[15].ToString());
                }
            }
        }

        private void ADCTestConsole2()
        {
            //  if (ADC_CH[16] > 5) CH16.BackColor = Color.Orange; // FAIL (CONSOLE TEST) Immobilizer Antenna (-)
            //  if (ADC_CH[16] > 5) CH16.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) Immobilizer Antenna (-)

            //  if (ADC_CH[17] > 5) CH17.BackColor = Color.Orange; // FAIL (CONSOLE TEST) Secondary Fused Battery (+) 23
            //  if (ADC_CH[17] > 5) CH17.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) Secondary Fused Battery (+) 23

            //  if (ADC_CH[18] > 5) CH18.BackColor = Color.Orange; // FAIL (CONSOLE TEST) LIN (11) Signal Rear Heated Seat Switch
            //  if (ADC_CH[18] > 5) CH18.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) LIN (11) Signal Rear Heated Seat Switch

            //  if (ADC_CH[19] > 5) CH19.BackColor = Color.Orange; // FAIL (CONSOLE TEST) Powertrain Sensor Bus Enable
            //  if (ADC_CH[19] > 5) CH19.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) Powertrain Sensor Bus Enable

            if (gv.LAMP.Contains("YELO"))
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[20] = gv.DiagnosticOverrideForm.BADC20Valid ? 0 : 10;
                if (ADC_CH[20] >= 3)
                {
                    CH20.BackColor = Color.Orange; // FAIL (CONSOLE TEST) Ignition Feed Console Lamp
                    gv.LAMP = "FAIL";
                    Trace.WriteLine("LAMP ADC_CH20 fail = " + ADC_CH[20].ToString());
                }

                if (ADC_CH[20] < 3)
                {
                    CH20.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) Ignition Feed Console Lamp
                    gv.LAMP = "PASS";
                    Trace.WriteLine("LAMP ADC_CH20 pass = " + ADC_CH[20].ToString());
                }
            }

            //  if (ADC_CH[21] > 5) CH21.BackColor = Color.Orange; // FAIL (CONSOLE TEST) LED Backlight Dimming Control
            //  if (ADC_CH[21] > 5) CH21.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) LED Backlight Dimming Control

            //  if (ADC_CH[22] > 5) CH22.BackColor = Color.Orange; // FAIL (CONSOLE TEST) Battery Feed ROW2 / Aux. USB Power
            //  if (ADC_CH[22] > 5) CH22.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) Battery Feed Row2 / Aux. USB Power

            //  if (ADC_CH[23] > 5) CH23.BackColor = Color.Orange; // FAIL (CONSOLE TEST) Ground Audio Jack Only
            //  if (ADC_CH[23] > 5) CH23.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) Ground Audio Jack Only
        }

        private void ADCTestConsole3()
        {
            //   if (ADC_CH[24] > 5) CH24.BackColor = Color.Orange; // FAIL (CONSOLE TEST) NOT USED
            //   if (ADC_CH[24] > 5) CH24.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) NOT USED

            //   if (ADC_CH[25] > 5) CH25.BackColor = Color.Orange; // FAIL (CONSOLE TEST) NOT USED
            //   if (ADC_CH[25] > 5) CH25.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) NOT USED

            //   if (ADC_CH[26] > 5) CH26.BackColor = Color.Orange; // FAIL (CONSOLE TEST) NOT USED
            //   if (ADC_CH[26] > 5) CH26.BackColor = Color.Aquamarine;  // PASS (HEADLINER TEST) NOT USED

            //   if (ADC_CH[27] <= 2) CH27.BackColor = Color.White; // FAIL (CONSOLE TEST) 12Vdc Power Outlet
            //   if (ADC_CH[27] >= 2) CH27.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST) 12Vdc Power Outlet

            if (gv.PEPS == "YELO")
            {
                if (gv.DiagnosticOverrideForm != null) ADC_CH[28] = gv.DiagnosticOverrideForm.BADC28Valid ? 0 : 10;
                if (ADC_CH[28] < 1) // PEPS ANTENNA TEST
                {
                    CH28.BackColor = Color.Orange;
                    gv.PEPS = "FAIL";
                    Trace.WriteLine("PEPS ADC_CH28 fail = " + ADC_CH[28].ToString());
                }

                if (ADC_CH[28] >= 1)
                {
                    CH28.BackColor = Color.Aquamarine;
                    gv.PEPS = "PASS";
                    Trace.WriteLine("PEPS ADC_CH28 pass = " + ADC_CH[28].ToString());
                }
            }
            //   if (ADC_CH[29] > 5) CH29.BackColor = Color.Orange; // FAIL (CONSOLE TEST)
            //   if (ADC_CH[29] > 5) CH29.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST)

            //   if (ADC_CH[30] > 5) CH30.BackColor = Color.Orange; // FAIL (CONSOLE TEST)
            //    if (ADC_CH[30] > 5) CH30.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST)

            //   if (ADC_CH[31] > 5) CH31.BackColor = Color.Orange; // FAIL (CONSOLE TEST)
            //   if (ADC_CH[31] > 5) CH31.BackColor = Color.Aquamarine;  // PASS (CONSOLE TEST)
        }

        private void ADCTestConsoleComplete()
        {
            Trace.WriteLine("BEGIN ADCTestConsoleComplete()");
            Trace.Indent();

            Trace.WriteLine("STOPPING ScanTimer");
            ScanTimer.Stop();
            Trace.WriteLine("Showing ConsoleTestResultForm");
            gv.ConsoleTestResultForm.Show();
            Trace.WriteLine("END ADCTestConsoleComplete()");
        }

        private void StopButton_Click(object sender, EventArgs e)
        {
            Trace.WriteLine("StopButton_Click()");
            ScanTimer.Stop();
            ExecuteButton.Enabled = true;
            StopButton.Enabled = false;
        }

        private void DiagnosticForm1_Load(object sender, EventArgs e)
        {
            Trace.WriteLine("DiagnosticForm1_Load()");
            textBoxfrm1VIN.Text = gv.VIN;

            if (labelHEADLINER_0_15.Visible)
            {
                labelCONSOLE_0_15.Visible = false;
            }

            _ = this.BeginInvoke((MethodInvoker)this.StartADCReadings);
        }

        private void buttonViewResults_Click(object sender, EventArgs e)
        {
            Trace.WriteLine("buttonViewResults_Click()");
            ScanTimer.Stop();

            if (PLC.Results.TesterName.Contains("HEADLINER"))
                gv.HeadlinerConnectTestForm.Show();
            else
            if (PLC.Results.TesterName.Contains("CONSOLE"))
                gv.ConsoleTestResultForm.Show();
        }
    }
}