﻿using AIOUSBNet;  // namespace exposes AIOUSB Class interface
using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Windows.Forms;

namespace Sample1
{
    public partial class HeadlinerTestForm : Form
    {
        private THLResultData ResultData = new THLResultData();

        public HeadlinerTestForm()
        {
            InitializeComponent();

            /*
			TComponentData ComponentData = new TComponentData(61, 0); //Office
			//TComponentData ComponentData = new TComponentData(61, 1000); //Production
			gv.CSN = ComponentData.CSN;
			*/
        }

        private static void GenerateHeadlinerTestResultFile()
        {
            //*******************************************************************************
            // ******************************************************************************
            // ******** Trial writing HEADLINER test result file for Larry to TX using Sharp7
            //*******************************************************************************
            //*******************************************************************************

            //string HL_strFullFIleName = "c:/Android/Data Directory/Default/HEADLINER_TESTER_1.txt";
            //string HL_strFullFIleName = "c:/Android/Data Directory/Default/HLTestLog." + DateTime.Now.ToString("s") + ".txt";
            string HL_strFullFIleName = "c:/Android/Data Directory/Default/HLTestLog." + PLC.Results.CSN + ".txt";

            StreamWriter HL_FileWriter = new StreamWriter(HL_strFullFIleName);

            HL_FileWriter.WriteLine("ETC Headlin 1,");  // Tester Name(13) ETC Console 2,
            HL_FileWriter.WriteLine("06/29/21,"); // Test Date(10)  MM / DD / YEAR,
            HL_FileWriter.WriteLine("07:19:00,"); // Cycle Start Time Stamp(8), Military HR.MN.SC,
            HL_FileWriter.WriteLine("07:19:45,"); // Cycle Complete Time Stamp(8), Military HR.MN.SC,
            HL_FileWriter.WriteLine("0045,"); // Total Cycle Time(seconds)(4), 9999,
            HL_FileWriter.WriteLine("0000000355149,"); // Build ID # (10),  0000000355149,
            HL_FileWriter.WriteLine("0000004364960,"); // CSN # (13), Sequence #	0000004364960,
            HL_FileWriter.WriteLine("1234,"); // Test 1 Mod Code(4), 1234,
            HL_FileWriter.WriteLine("Test Channel 1 * *****,"); // Test 1 Channel Name(20),	Test Channel 1 * *****,
            HL_FileWriter.WriteLine("PASS,"); // Test 1 Result(4), PASS,
            HL_FileWriter.WriteLine("12.00,"); // Test 1 Data Volts DC(5), 	12.00,
            HL_FileWriter.WriteLine(">= 12.00,"); // Test 1 PASS Voltage Limit >= 12.00,
            HL_FileWriter.WriteLine("<= 12.00,"); // Test 1 FAIL Voltage Limit <= 12.00,
            HL_FileWriter.WriteLine("******************************,");  // Test 1 Result Comments(30),		******************************,
            HL_FileWriter.WriteLine("5678,");   // Test 2 Mod Code(4),	5678,
            HL_FileWriter.WriteLine("Test Channel 2 * *****,");  // Test 2 Channel Name(20),	Test Channel 2 * *****,
            HL_FileWriter.WriteLine("PASS,");   // Test 2 Result(4), PASS,
            HL_FileWriter.WriteLine("12.00,");   // Test 2 Data Volts DC(5), 12.00,
            HL_FileWriter.WriteLine(">= 12.00,");   // Test 2 PASS Voltage Limit >= 12.00,
            HL_FileWriter.WriteLine("<= 12.00,");   // Test 2 FAIL Voltage Limit <= 12.00,
            HL_FileWriter.WriteLine("******************************,");   // Test 2 Result Comments(30), 	******************************,
            HL_FileWriter.Close();
        }

        private void HeadlinerConnectTestForm_Load(object sender, EventArgs e)  // FORM LOAD ALSO CHECKS THE PIP SWITCHES WHICH STARTS THE DRAW DOWN SOLENOIDS
        {
            Trace.WriteLine("BEGIN HeadlinerConnectTestForm_Load ");
            Trace.Indent();

            Trace.WriteLine("Ejecting all");
            Eject55Way();
            Eject16WayX1AndX2();
            Eject59Way();

            OperatorInstructions.Text = ""; // CLEAR SCREEN

            if (gv.RETEST.Contains(""))
            {
                Trace.WriteLine("RETEST contains '' ");
                textBoxfrm4CSN.Text = PLC.Results.CSN;

                OperatorInstructions.Text = ""; // CLEAR SCREEN
                OperatorInstructions.Text = "<<< OPERATOR >>> \n\r\n\r\n\r";
                OperatorInstructions.Text += "OPEN L&R SUNSHADE LAMPS\n\r\n\r";
                OperatorInstructions.Text += "CONNECT 59-WAY OR 55-WAY \n\r\n\r";
                // OperatorInstructions.Text += "CONNECT 16-WAY (X1 & X2) \n\r\n\r"; // Plant wants this probe removed 
                textBox2.Text = ".";


                //*********************************************
                // ***** PREMIUM MANIFEST FOR RED RABBIT TEST
                //****** MUST REMARK OUT FOR PRODUCTION
                //********************************************
                /*
                // HEADLINER MANIFEST - Highline
                ResultData.Clear(); // Clear Screen
                ResultData.Head0 = "0000000355149";
                ResultData.Head1 = "103xxxxxxxxxx";
                ResultData.HeadCSN = "0000004364960";
                ResultData.HeadModcode[501] = "84874628FT28";
                //ResultData.HeadModcode[201] = "84810967M38A";
                ResultData.HeadModcode[301] = "84793091FT29";
                ResultData.HeadModcode[901] = "84660808A28B";
                ResultData.HeadModcode[601] = "????????????";
                ResultData.HeadModcode[602] = "????????????";
                ResultData.HeadModcode[603] = "????????????";
                ResultData.HeadModcode[604] = "????????????";
                ResultData.HeadModcode[601] = "85116495M40P";
                ResultData.HeadModcode[801] = "846608188A28A";
                ResultData.HeadModcode[1801] = "84798058FW80";
                */

                gv.HeadlinerTestResultsOptionsForm.Manifest_Options.Text = gv.ConsoleTestMainForm.RetrieveManifestFromPLC();

                // PARSE OUT THE CSN #
                PLC.Results.CSN = ResultData.HeadCSN;
                textBoxfrm4CSN.Text = PLC.Results.CSN;
                InitializePartNumberTextBoxes();
            }
            Trace.WriteLine("Waiting for Start from PLC ");
            while (!(PLC.bGetStart() || cbFakePLCStart.Checked)) Application.DoEvents();
            Trace.WriteLineIf(cbFakePLCStart.Checked, "Got Start from PLC");

            Trace.WriteLine("starting Timer1");
            timer1.Start();

            Trace.Unindent();
            Trace.WriteLine("END HeadlinerConnectTestForm_Load");
        }

        private void InitializePartNumberTextBoxes()
        {
            Trace.WriteLine("BEGIN InitializePartNumberTextBoxes");
            Trace.Indent();
            // *******************************************************
            // ********* HEADLINER TEST MOD codes ********************
            // *******************************************************
            int iModcode;

            //// Search Manifest for Modcode <<<< 201 THROUGH 205 >>>> (Cell Phone MIC LEFT ??)
            gv.CMIC = "GRAY";
            for (iModcode = 201; iModcode <= 205; ++iModcode)
                if (ResultData.HeadModcode[iModcode] != "")
                {
                    Trace.WriteLine("Manifest contains CMIC as "+iModcode.ToString());
                    gv.CMIC = "YELO";
                    break;
                }

            //Search Manifest for Modcode <<<< 301 through 307 >>>> (Other MIC RIGHT ??)
            gv.MIC = "GRAY";
            for (iModcode = 301; iModcode <= 307; ++iModcode)
                if (ResultData.HeadModcode[iModcode] != "")
                {
                    Trace.WriteLine("Manifest contains MIC as " + iModcode.ToString());
                    gv.MIC = "YELO";
                }

            // Search Manifest for Modcode (Noise Cancel MIC)
            gv.ANCMIC = (ResultData.HeadModcode[1801] != "") ? "YELO" : "GRAY";
            Trace.WriteLineIf(gv.ANCMIC == "YELO", "Manifest contains ANCMIC as 1801");

            // Search Manifest for Modcode <<<< 501 TO 510 >>>>
            //
            // (DOME LAMP)
            //
            // WHICH IS WHICH ???????????
            //
            // LAMP_H = STOP LAMP
            // ECLAMP = EXTERIOR COURTESY LAMP
            // R2LAMPS = ROW 2 DOME LAMP SWITCH
            // R2LAMPS2 = ROW 2 DOME LAMP SWITCH 2
            // R2LAMPC = ROW 2 DOME LAMP CONTROL <<<< VALIDATED AS 501 THROUGH 510 *********************
            // R2LAMPC2 = ROW 2 DOME LAMP CONTROL2

            // HEADLINER LAMP
            gv.R2LAMPC = "GRAY";
            for (iModcode = 501; iModcode <= 510; ++iModcode) if (ResultData.HeadModcode[iModcode] != "")
                {
                    Trace.WriteLine("Manifest contains R2LAMPC as " + iModcode.ToString());
                    gv.R2LAMPC = "YELO";
                    break;
                }

            // Search Manifest for Modcode <<<< 801 TO 830 >>>> (LEFT SUNSHADE LAMP)
            gv.LSLAMP = "GRAY";
            for (iModcode = 801; iModcode <= 830; ++iModcode) if (ResultData.HeadModcode[iModcode] != "")
                {Trace.WriteLine("Manifest contains LSLAMP as " + iModcode.ToString());
                    gv.LSLAMP = "YELO";
                    break;
                }

            // Search Manifest for Modcode <<<< 901 TO 930 >>>> (RIGHT SUNSHADE LAMP)
            gv.RSLAMP = "GRAY";
            for (iModcode = 901; iModcode <= 930; ++iModcode) if (ResultData.HeadModcode[iModcode] != "")
                {
                    Trace.WriteLine("Manifest contains RSLAMP as " + iModcode.ToString());
                    gv.RSLAMP = "YELO";
                    break;
                }
            Trace.Unindent();
            Trace.WriteLine("END InitializePartNumberTextBoxes");
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            PLC.Heartbeat();
            OperatorInstructions.Text = ""; // CLEAR SCREEN
                                            //gv.DiagnosticForm.GotFocus();
                                            // CHECK FOR RETRY FAILED TEST
            if (gv.MIC.Contains("FAIL")) // RETURN TO PROMPT OPERATOR TO CHECK CONNECTION
            {
                gv.RETEST = "YES";
                OperatorInstructions.Text += "CHECK FOR OPEN LEFT MIC\n\r\n\r";
                ResultData.bCheckForOpenLeftMic = true;
            }

            if (gv.CMIC.Contains("FAIL")) // RETURN TO PROMPT OPERATOR TO CHECK CONNECTION
            {
                gv.RETEST = "YES";
                OperatorInstructions.Text += "CHECK FOR OPEN RIGHT MIC\n\r\n\r";
                ResultData.bCheckForOpenRightMic = true;
            }

            if (gv.LSLAMP.Contains("FAIL")) // RETURN TO PROMPT OPERATOR TO CHECK CONNECTION
            {
                gv.RETEST = "YES";
                OperatorInstructions.Text += "MAKE SURE LEFT SUNSHADE LAMP IS ON\n\r\n\r";
                ResultData.bMakeSureLeftSunshadeLampIsOn = true;
            }

            if (gv.RSLAMP.Contains("FAIL")) // RETURN TO PROMPT OPERATOR TO CHECK CONNECTION
            {
                gv.RETEST = "YES";
                OperatorInstructions.Text += "MAKE SURE RIGHT SUNSHADE LAMP IS ON\n\r\n\r";
                ResultData.bMakeSureRightSunshadeLampIsOn = true;
            }

            if (gv.R2LAMPC.Contains("FAIL")) // RETURN TO PROMPT OPERATOR TO CHECK CONNECTION
            {
                gv.RETEST = "YES";
                OperatorInstructions.Text += "CHECK FOR OPEN DOME LAMP\n\r\n\r";
                ResultData.bCheckForOpenDomeLamp = true;
            }

            if (true)
            //if (gv.RETEST.Contains(""))
            {
                // ONE SHOT
                if (ResultData.bConnect16_WayX1X2)
                {
                    textBox2.Text = "";
                }
                else
                {
                    OperatorInstructions.Text = ""; // CLEAR SCREEN
                    OperatorInstructions.Text = "<<< OPERATOR >>> \n\r\n\r\n\r";
                    OperatorInstructions.Text += "OPEN L&R SUNSHADE LAMPS\n\r\n\r";
                    OperatorInstructions.Text += "CONNECT 59-WAY OR 55-WAY \n\r\n\r";
               //     OperatorInstructions.Text += "CONNECT 16-WAY (X1 & X2) \n\r\n\r";  // Plant wants this probe removed
                    //refreshed the operator prompt message
                }

                byte[] data = new byte[] { 0, 0, 0, 0 };

                _ = AIOUSB.DIO_ReadAll(gv.diDIO, data);

                bool bConsole55WayDetected = ((data[2] & 0x02) == 0x02);// Console 55-way PIP Switch or Headliner 59-way PIP Switch
                bool bHeadliner59WayDetected = bConsole55WayDetected;// Console 55-way PIP Switch or Headliner 59-way PIP Switch
                bool bHeadliner55WayDetected = ((data[2] & 0x04) == 0x04);// Headliner 55-way PIP Switch
               // bool bHeadliner16WayX1Detected = ((data[2] & 0x08) == 0x08);// Headliner 16-way X1 PIP Switch
               // bool bHeadliner16WayX2Detected = ((data[2] & 0x10) == 0x10);// Headliner 16-way X2 PIP Switch

                if (gv.J2HDebugMode) // debugging mode only!!! -- J2H DEBUG
                {
                    Trace.WriteLine("Debug overrides used for 55Way & 59Way");
                    bConsole55WayDetected = gv.DiagnosticOverrideForm.DI1Detected;
                    bHeadliner59WayDetected = gv.DiagnosticOverrideForm.DI1Detected;
                    bHeadliner55WayDetected = gv.DiagnosticOverrideForm.DI2Detected;
                //    bHeadliner16WayX1Detected = gv.DiagnosticOverrideForm.DI3Detected;
                //    bHeadliner16WayX2Detected = gv.DiagnosticOverrideForm.DI4Detected;
                }

                Detect59Way(bHeadliner59WayDetected);
                Detect55Way(bHeadliner55WayDetected);
            //    Detect16Ways(bHeadliner16WayX1Detected, bHeadliner16WayX2Detected);

                {
                    // TEST FOR MANUAL RELEASE BUTTON PRESSED ON 59 WAY
                    if (!bHeadliner59WayDetected)
                    {
                        if (ResultData.b59_WayConnected)
                        {
                            PLC.SendTestComplete();
                            Eject59Way();
                            Eject16WayX1AndX2();
                        }
                    }

                    // TEST FOR MANUAL RELEASE BUTTON PRESSED ON 55 WAY
                    if (!bHeadliner55WayDetected)
                    {
                        if (ResultData.b55_WayConnected)
                        {
                            PLC.SendTestComplete();
                            // RELEASE BOTH 55-WAY AND 16/16-WAY UNLATCH AND BLOW OFF SINCE IT HAS NO PUSH BUTTON
                            Eject55Way();
                            Eject16WayX1AndX2();
                        }
                    }
                }
                //***********************************************************************************************
                //********* Read Module Part Numbers *************************************************************
                //************************************************************************************************
                // Shell Command to Launch Vehicle Spy ***********************************************************
                //************************************************************************************************
                // TCPIP TEXT COMMAND APP TO VEHICLE SPY TO START VEHICLE SPY PART NUMBER READ TEST BLOCK ROUTINES
                // ***********************************************************************************************

                // *****************************************************
                //********* Switch Form to Start Analog Test ***********
                //******************************************************

                if (bHeadliner59WayDetected || bHeadliner55WayDetected)
                {
                   // if (bHeadliner16WayX1Detected)
                   // {
                    //    if (bHeadliner16WayX2Detected)
                     //   {
                            OperatorInstructions.Text = "All connectors detected\n\r\nProceeding with ADC tests...\r\n";
                            timer1.Enabled = false;
                            for (int i = 0; i < 10; i++)
                            {
                                Application.DoEvents();
                                Thread.Sleep(100);
                            }
                            GenerateHeadlinerTestResultFile();
                            gv.DiagnosticForm.Show();
                       // }
                   // }
                }
            }
        }

        private void Detect59Way(bool bHeadliner59WayDetected)
        {
            Trace.WriteLine("Detect59Way(" + bHeadliner59WayDetected.ToString()+")");
            if (bHeadliner59WayDetected)
            {
                // ONE SHOT
                if (ResultData.b59_WayConnected)
                {
                    // DO NOTHING
                }
                else
                {
                    labelPIPswitchHDL59A.BackColor = Color.Aquamarine;  // Operator Display Change 59-way PIPa switch indicator COLOR from Default YELLOW to AQUAMARINE
                    labelPIPswitchHDL59B.BackColor = Color.Aquamarine;  // Operator Display Change 59-way PIPb switch indicator COLOR from Default YELLOW to AQUAMARINE

                    labelDrawDownHDL59.BackColor = Color.Aquamarine;  // Operator Display Change 59-way DrawDown indicator COLOR from Default YELLOW to AQUAMARINE

                    DrawDownAndLatch59Way();

                    labelLatchHDL59A.BackColor = Color.Aquamarine;  //
                    labelLatchHDL59B.BackColor = Color.Aquamarine;  //
                    buttonConnect_59_Way.BackColor = Color.Aquamarine;  //

                    ResultData.b59_WayPIPSwitchesDetected = true;
                    ResultData.b59_WayDrawdownSolenoidEnergized = true;
                    ResultData.b59_WayLatchSolenoidEnergized = true;
                    ResultData.b59_WayConnected = true;
                }
            }
        }

        private void Detect55Way(bool bHeadliner55WayDetected)
        {
            Trace.WriteLine("Detect55Way(" + bHeadliner55WayDetected.ToString() + ")");
            if (bHeadliner55WayDetected)
            {
                // ONE SHOT
                if (ResultData.b55_WayConnected)
                {
                    // DO NOTHING
                }
                else
                {
                    labelPIPswitchHDL55A.BackColor = Color.Aquamarine; // Operator Display Change 55-way PIPa switch indicator COLOR from Default YELLOW to AQUAMARINE
                    labelPIPswitchHDL55B.BackColor = Color.Aquamarine; // Operator Display Change 55-way PIPb switch indicator COLOR from Default YELLOW to AQUAMARINE

                    DrawDownAndLatch55Way();

                    labelDrawDownHDL55.BackColor = Color.Aquamarine; // Operator Display Change 55-way DrawDown indicator COLOR from Default YELLOW to AQUAMARINE
                    labelLatchHDL55.BackColor = Color.Aquamarine; //
                    buttonConnect_55_Way.BackColor = Color.Aquamarine; //

                    ResultData.b55_WayPIPSwitchesDetected = true;
                    ResultData.b55_WayDrawdownSolenoidEnergized = true;
                    ResultData.b55_WayLatchSolenoidEnergized = true;
                    ResultData.b55_WayConnected = true;
                }
            }
        }

        private void Detect16Ways(bool bHeadliner16WayX1Detected, bool bHeadliner16WayX2Detected)
        {
            Trace.WriteLine("Detect16Way(" + bHeadliner16WayX1Detected.ToString() + "," + bHeadliner16WayX2Detected.ToString() + ")");
            if (bHeadliner16WayX1Detected)
            {
                labelPIPswitchHDL16X1.BackColor = Color.Aquamarine; // Operator Display Change 16/16 - way X1 PIP switch indicator COLOR from Default YELLOW to AQUAMARINE
                Label_16_16_X1.BackColor = Color.Aquamarine; //
            }

            if (bHeadliner16WayX2Detected)
            {
                labelPIPswitchHDL16X2.BackColor = Color.Aquamarine; // Operator Display Change 16/16 - way X2 PIP switch indicator COLOR from Default YELLOW to AQUAMARINE
                Label_16_16_X2.BackColor = Color.Aquamarine; //
            }

            if (bHeadliner16WayX1Detected && bHeadliner16WayX2Detected)
            {
                buttonConnect_16_16_way.BackColor = Color.Aquamarine; //
                ResultData.b16_16_WayX1X2Connected = true;
            }
        }

        public static void Eject55Way()
        {
            Trace.WriteLine("Eject55Way()");
            _ = AIOUSB.DIO_Write1(gv.diDIO, 10, 1); // Turn OFF SOL #2   HEADLINER LATCH BH#2 55-WAY
            Thread.Sleep(100);
            _ = AIOUSB.DIO_Write1(gv.diDIO, 12, 1); // Turn OFF SOL #4   HEADLINER DRAW DOWN BH #2 55-WAY
        }

        public static void Eject59Way()
        {
            Trace.WriteLine("Eject59Way()");
            _ = AIOUSB.DIO_Write1(gv.diDIO, 11, 1); // Turn OFF SOL #3   HEADLINER DRAW DOWN BH #1 59-WAY
            Thread.Sleep(100);
            _ = AIOUSB.DIO_Write1(gv.diDIO, 9, 1);  // Turn OFF SOL #1   HEADLINER LATCH BH #1 59-WAY
        }

        public static void Eject16WayX1AndX2()
        {
            Trace.WriteLine("Eject16WayX1AndX2()");
            _ = AIOUSB.DIO_Write1(gv.diDIO, 13, 0); // TURN ON SOL #5   HEADLINER AIR EJECT BH #3 16/16-WAY
            Thread.Sleep(1000); // Todo: shove into background thread / task / await / async
            _ = AIOUSB.DIO_Write1(gv.diDIO, 13, 1); // TURN ON SOL #5   HEADLINER AIR EJECT BH #3 16/16-WAY
        }

        public static void DrawDownAndLatch55Way()
        {
            Trace.WriteLine("DrawDownAndLatch55Way()");
            _ = AIOUSB.DIO_Write1(gv.diDIO, 12, 0); // Energize Sol #4   HEADLINER DRAW DOWN BH #2 55-way
            Thread.Sleep(700);
            _ = AIOUSB.DIO_Write1(gv.diDIO, 10, 0); // Turn ON SOL #2 HEADLINER LATCH BH #2 55-WAY
        }

        public static void DrawDownAndLatch59Way()
        {
            Trace.WriteLine("DrawDownAndLatch59Way()");
            _ = AIOUSB.DIO_Write1(gv.diDIO, 11, 0); // Energize Sol #3   HEADLINER DRAW DOWN BH #1 59-way
            Thread.Sleep(700);
            _ = AIOUSB.DIO_Write1(gv.diDIO, 9, 0);  // Turn ON SOL #1 HEADLINER LATCH BH#1 59-WAY
        }

        private void label6_Click(object sender, EventArgs e)
        {
            Trace.WriteLine("label6 clicked");
            // Initialize all Outputs to the OFF State

            _ = AIOUSB.DIO_Write1(0, 9, 1);  // Turn OFF SOL #1   CONSOLE 55-way LATCH / HEADLINER LATCH BH#1
            _ = AIOUSB.DIO_Write1(0, 10, 1); // Turn OFF SOL #2   CONSOLE 55-way DRAW DOWN / HEADLINER LATCH BH#2 59-WAY
            _ = AIOUSB.DIO_Write1(0, 11, 1); // Turn OFF SOL #3   HEADLINER DRAW DOWN BH #1 59-WAY
            _ = AIOUSB.DIO_Write1(0, 12, 1); // Turn OFF SOL #4   HEADLINER DRAW DOWN BH #2 55-WAY
            _ = AIOUSB.DIO_Write1(0, 13, 1); // TURN OFF SOL #5   HEADLINER AIR EJECT BH #3 16/16-WAY

            _ = AIOUSB.DIO_Write1(0, 9, 1);  // Turn OFF SOL #1   CONSOLE 55-way LATCH / HEADLINER LATCH BH#1
            _ = AIOUSB.DIO_Write1(0, 10, 1); // Turn OFF SOL #2   CONSOLE 55-way DRAW DOWN / HEADLINER LATCH BH#2
            _ = AIOUSB.DIO_Write1(0, 11, 1); // Turn OFF SOL #3   HEADLINER DRAW DOWN BH #1
            _ = AIOUSB.DIO_Write1(0, 12, 1); // Turn OFF SOL #4   HEADLINER DRAW DOWN BH #2
            _ = AIOUSB.DIO_Write1(0, 13, 1); // TURN OFF SOL #5   HEADLINER AIR EJECT BH #3

            _ = AIOUSB.DIO_Write1(0, 9, 1);  // Turn OFF SOL #1   CONSOLE 55-way LATCH / HEADLINER LATCH BH#1
            _ = AIOUSB.DIO_Write1(0, 10, 1); // Turn OFF SOL #2   CONSOLE 55-way DRAW DOWN / HEADLINER LATCH BH#2
            _ = AIOUSB.DIO_Write1(0, 11, 1); // Turn OFF SOL #3   HEADLINER DRAW DOWN BH #1
            _ = AIOUSB.DIO_Write1(0, 12, 1); // Turn OFF SOL #4   HEADLINER DRAW DOWN BH #2
            _ = AIOUSB.DIO_Write1(0, 13, 1); // TURN OFF SOL #5   HEADLINER AIR EJECT BH #3
        }

        private void ButtonExit_Click(object sender, EventArgs e)
        {
            Trace.WriteLine("ButtonExit clicked");
            gv.bCloseProgram = true; // TODO: only set to true if developer means it!!
            if (gv.VSpy3ProcessHandle != null)
                _ = gv.VSpy3ProcessHandle.CloseMainWindow();
            Close();
        }

        private void ButtonAbortTest_Click(object sender, EventArgs e)
        {
            Trace.WriteLine("ButtonAbortTest clicked");
            timer1.Stop();
            PLC.SendTestFailed();
            gv.HeadlinerTestResultsOptionsForm.Show();
        }
    }
}