﻿

using System;
using System.Windows.Forms;
using System.Diagnostics;
using System.Runtime.CompilerServices;
using AIOUSBNet;




namespace Sample1
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Trace.TraceInformation("Main");
            Trace.Indent();

            gv.J2HDebugMode = true;
            //gv.J2HDebugMode = false;

            //gv.bETC_PLC = false; // production PLC at facility
            gv.bETC_PLC = true; // borrowed PLC at ETC lab

            //ResultData = new TResultData();

            //****************************************************************************************
            //*********** FORT WAYNE INDIANA TESTER TYPE "CONSOLE" OR "HEADLINER" ********************
            //****************************************************************************************
            gv.SOFTWARE_VERSION = "FW Software Ver. (1.0.0) 7/7/21";

            //select either test type by adding a slash to this line's start "//*" vs "/*"
            //PLC.Results.TesterName = "HEADLINER";
            PLC.Results.TesterName = "CONSOLE";
            

            Trace.WriteLine(PLC.Results.TesterName+" Test -- " + gv.SOFTWARE_VERSION);
            Trace.WriteLineIf(gv.J2HDebugMode, "Running in Simulated Mode");
            Trace.WriteLineIf(gv.bETC_PLC, "Operating PLC in 0x0000 mode - demo PLC only");

            Trace.Indent();

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            if (DetectADCandDIODeviceIndex() != 0 /* ERROR_SUCCES*/ )
            {
                Trace.TraceError("ERROR! Required sensor hardware is missing!");
                //handle either or both ADC/DIO devices not found
            }

            

            //TResultData ResultData
            while (!gv.bCloseProgram)
            {
                Trace.WriteLine("Starting Testing");
                Trace.Indent();

                if (gv.J2HDebugMode)
                {
                    gv.DiagnosticOverrideForm = new DiagnosticOverrides();
                    gv.DiagnosticOverrideForm.Show();
                    gv.DiagnosticOverrideForm.TopMost = true;
                }

                if (PLC.Results.TesterName.Contains("CONSOLE"))
                {
                    gv.VSpy3ProcessHandle = null;

                    if (gv.ConsoleTestMainForm != null) gv.ConsoleTestMainForm.Close();
                    gv.ConsoleTestMainForm = new ConsoleTestForm3();
                    //gv.ConsoleTestMainForm.TopMost = true;

                    if (gv.DiagnosticForm != null) gv.DiagnosticForm.Close();
                    gv.DiagnosticForm = new DiagnosticForm();
                    //gv.DiagnosticForm.TopMost = true;


                    if (gv.ConsoleTestResultForm != null) gv.ConsoleTestResultForm.Close();
                    gv.ConsoleTestResultForm = new ConsoleTestResultsForm();
                    //gv.ConsoleTestResultForm.TopMost = true;

                    if (gv.CSNEntryForm != null) gv.CSNEntryForm.Close();
                    gv.CSNEntryForm = new CSNEntryForm();
                    gv.CSNEntryForm.buttonConsoleTest.Visible = true;
                    //gv.CSNEntryForm.TopMost = true;

                    Application.Run(gv.ConsoleTestMainForm); // 3rd Console Test Connectors
                }

                if (PLC.Results.TesterName.Contains("HEADLINER"))
                {
                    if (gv.HeadlinerConnectTestForm != null) gv.HeadlinerConnectTestForm.Close();
                    gv.HeadlinerConnectTestForm = new HeadlinerTestForm();// 3rd Console Test Connectors
                    //gv.HeadlinerConnectTestForm.TopMost = true;

                    if (gv.DiagnosticForm != null) gv.DiagnosticForm.Close();
                    gv.DiagnosticForm = new DiagnosticForm();// 5th  Analog Test Validation
                    //gv.DiagnosticForm.TopMost = true;

                    if (gv.HeadlinerTestResultsOptionsForm != null) gv.HeadlinerTestResultsOptionsForm.Close();
                    gv.HeadlinerTestResultsOptionsForm = new HeadlinerTestResultsOptionsForm();// 2nd Console Test Option Selection / Test Results
                    //gv.HeadlinerTestResultsOptionsForm.TopMost = true;

                    if (gv.CSNEntryForm != null) gv.CSNEntryForm.Close();
                    gv.CSNEntryForm = new CSNEntryForm();// 1st VIN Keypad
                    gv.CSNEntryForm.buttonHeadlinerTest.Visible = true;
                    //gv.CSNEntryForm.TopMost = true;

                    Application.Run(gv.HeadlinerConnectTestForm); 
                }
            }
        }
        static uint DetectADCandDIODeviceIndex()
        {
            Trace.WriteLine("BEGIN DetectADCandDIODeviceIndex()");
            Trace.Indent();

            _ = AIOUSB.AIOUSB_ReloadDeviceLinks(); // redetects devices; generally just for convenience; do this first as DeviceIndices can change
            gv.diDIO = AIOUSB.diNone;
            uint diMask = AIOUSB.GetDevices();
            for (uint DeviceIndex = 0; DeviceIndex < 32; DeviceIndex++)
            {
                if ((diMask & (1 << (int)(DeviceIndex))) > 0)
                {
                    uint pPID;
                    _ = AIOUSB.QueryDeviceInformation(DeviceIndex, out pPID, out _, out _, out _, out _);
                    // detect ADC devices
                    if (((pPID >= 0x8040) && (pPID < 0x805F))
                         || ((pPID >= 0x8140) && (pPID < 0x815F)))
                    {
                        gv.diADC = DeviceIndex;
                        Trace.WriteLine("ACCES DAQ-PACK ["+pPID.ToString()+"] detected as DeviceIndex "+DeviceIndex.ToString());
                        continue;
                    }
                    // detect USB-IIRO-16 devices
                    if ((pPID >= 0x8010) && (pPID <= 0x801F))
                    {
                        Trace.WriteLine("ACCES IIRO [" + pPID.ToString() + "] detected as DeviceIndex " + DeviceIndex.ToString());
                        gv.diDIO = (uint)DeviceIndex;
                        continue;
                    }
                }
            }

            Trace.Unindent();
            Trace.WriteLine("END DetectADCandDIODeviceIndex()");
            if ((gv.diADC != AIOUSB.diNone) && (gv.diDIO != AIOUSB.diNone))
                return 1;
            return 0; /* ERROR_SUCCESS */
        }
    }
}
